/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.base.controller;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.base.dto.UserOnlineReportDto;
import com.tofly.base.entity.UserLog;
import com.tofly.base.service.UserLogService;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.oauth.util.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.security.Security;


/**
 * 日志信息表
 *
 * @author admin
 * @date Thu Oct 10 00:00:00 CST 2019
 */
@RestController
@RequestMapping("/userLog")
@Api(tags = "用户在线日志记录")
@Slf4j
public class UserLogController {

    @Autowired
    private UserLogService userLogService;

    /**
     * 分页查询
     *
     * @param page 分页对象
     * @param userLog  日志信息表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getLogPage(Page page, @ModelAttribute UserLog userLog) {

        return ResultRespone.success(userLogService.page(page, Wrappers.query(userLog)));
    }

    /**
     * 新增用户在线日志记录
     *
     * @param userLog 新增用户在线日志记录
     * @return ResultRespone
     */
    //@ToFlyAppLog(title="新增日志信息表"
    @ApiOperation(value = "新增用户在线日志记录")
    @PostMapping()
    public ResultRespone save(@RequestBody UserLog userLog) {
        if(SecurityUtils.getUser()==null){
            ResultRespone response = new ResultRespone();
            response.setCode(401);
            response.setMessage("token失效");
            return response;
        }
        userLog.setUserId(SecurityUtils.getUser().getUserid());
        return ResultRespone.success(userLogService.saveData(userLog));
    }

    /**
     * 用户统计报表
     *
     * @param dto 用户统计报表
     * @return ResultRespone
     */
    //@ToFlyAppLog(title="新增日志信息表"
    @ApiOperation(value = "用户统计报表")
    @GetMapping("/userOnlineReport")
    public ResultRespone getUserOnlineReport(UserOnlineReportDto dto) {
        return ResultRespone.success(userLogService.userOnlineReport(dto));
    }

}
