package com.tofly.xrtygis.commons;


import lombok.Data;

import java.time.*;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author aiyuan
 * @date 2021/7/27
 */
public class DateWeekUtil {


    /**
     *获取当月第一天
     */
    public static LocalDateTime getFirstLocalDayOfMonth(LocalDateTime localDateTime) {
        return localDateTime.with(TemporalAdjusters.firstDayOfMonth()).with(LocalTime.MIN);
    }

    /**
     *获取当月第一周  以第一个周一为准
     */
    private static LocalDateTime getFirstMonday(LocalDateTime sourceTime) {
        LocalDateTime firstMondayOfMonth = getFirstLocalDayOfMonth(sourceTime);
        for (int i = 0; i < 6; i++) {
            DayOfWeek dayOfWeekTemp = firstMondayOfMonth.getDayOfWeek();
            if (dayOfWeekTemp.equals(DayOfWeek.MONDAY)) {
                break;
            }
            //往后推一天
            firstMondayOfMonth = firstMondayOfMonth.plusDays(1);
        }
        return firstMondayOfMonth;
    }


    /**
     * 获取几月份的第几周 [X月份第X周]
     *
     */
    public static String getMonthNoAndWeekNo(LocalDateTime sourceTime) {
        String monthNoAndWeekNo;
        //获取当月的第一天
        //获取月第一个周一,从当月第一天开始找

        LocalDateTime firstMondayOfMonth = getFirstMonday(sourceTime);

        //比较当月的第一个星期一 < = 参数时间
        if (firstMondayOfMonth.isBefore(sourceTime) || firstMondayOfMonth.isEqual(sourceTime)) {
            //当月第一个周一在当前时间之前 firstMondayOfMonth<=sourceTime
            //计算两个时间间隔天数
            int dayOfMonthFirstMonday = firstMondayOfMonth.getDayOfMonth();
            int dayOfMonthSourceTime = sourceTime.getDayOfMonth();

            int diffDays = dayOfMonthSourceTime - dayOfMonthFirstMonday;
            //第几周weekNo
            int weekNo = (diffDays / 7) + 1;
            //月份
            int monthNo = sourceTime.getMonth().getValue();

            monthNoAndWeekNo = monthNo + "月" + "第" + weekNo + "周";
        } else {
            //如果当月的第一个周一大于参数时间,则要计算到上个月份去
            //获取上一个月的第一个周一
            LocalDateTime lastMontLocalDateTime = sourceTime.minusMonths(1);
            //上个月的第一天
            //从上个月的第一天开始找周一
            LocalDateTime firstMondayOfMonthLast = getFirstMonday(lastMontLocalDateTime);

            //  计算两个时间间隔天数 (上月第一个周一 减去 当前时间)
            Duration duration = Duration.between(firstMondayOfMonthLast, sourceTime);
            long diffDays = duration.toDays(); //相差的天数
            //第几周weekNo
            long weekNo = (diffDays / 7) + 1;
            //月份
            int monthNo = firstMondayOfMonthLast.getMonth().getValue();//汉字版月份

            monthNoAndWeekNo = monthNo + "月" + "第" + weekNo + "周";
        }

        return monthNoAndWeekNo;
    }

    /**
     * Date转LocalDateTime
     *
     */
    public static LocalDateTime convertToLocalDateTime(Date date){
        Instant instant = date.toInstant();
        ZoneId zoneId = ZoneId.of("GMT+8");
        LocalDateTime startOfWeek = LocalDateTime.ofInstant(instant,zoneId)
                .withHour(date.getHours())
                .withMinute(date.getMinutes())
                .withSecond(date.getSeconds())
                .withNano(0);
        return startOfWeek;
    }
    /**
     * 根据指定日期取每周周一的日期，获取当前日期往前推多少周数
     *
     */
    public static List<Date> getFirstWeeksDate(int weeks,Date date) {
        Instant instant = date.toInstant();
        ZoneId zoneId = ZoneId.of("GMT+8");
        List<Date> list=new ArrayList<>();

        for (int i = 0; i < weeks; i++) {
            LocalDateTime startOfWeek = LocalDateTime.ofInstant(instant,zoneId)
                    .minusWeeks(i)
                    .with(DayOfWeek.MONDAY)
                    .withHour(0)
                    .withMinute(0)
                    .withSecond(0)
                    .withNano(0);
            ZonedDateTime zonedStartOfWeek = startOfWeek.atZone(zoneId);

            /*LocalDateTime endOfWeek = startOfWeek
                    .with(DayOfWeek.SUNDAY)
                    .withHour(23)
                    .withMinute(59)
                    .withSecond(59)
                    .withNano(999999999);
            ZonedDateTime zonedEndOfWeek = endOfWeek.atZone(zoneId);*/
            list.add(Date.from(zonedStartOfWeek.toInstant()));
        }

        return list;
    }

    @Data
    public static class DateRange {

        private Date dateFrom;
        private Date dateTo;
    }
    public static List<DateRange> getLastWeeksDateRanges(int weeks) {
        List<DateRange> dateRanges = new ArrayList<>();
        ZoneId zoneId = ZoneId.of("GMT+8");

        for (int i = 1; i <= weeks; i++) {
            LocalDateTime startOfWeek = LocalDateTime.now(zoneId)
                    .minusWeeks(i)
                    .with(DayOfWeek.MONDAY)
                    .withHour(0)
                    .withMinute(0)
                    .withSecond(0)
                    .withNano(0);
            ZonedDateTime zonedStartOfWeek = startOfWeek.atZone(zoneId);

            LocalDateTime endOfWeek = startOfWeek
                    .with(DayOfWeek.SUNDAY)
                    .withHour(23)
                    .withMinute(59)
                    .withSecond(59)
                    .withNano(999999999);
            ZonedDateTime zonedEndOfWeek = endOfWeek.atZone(zoneId);

            DateRange dateRange = new DateRange();
            dateRange.setDateFrom(Date.from(zonedStartOfWeek.toInstant()));
            dateRange.setDateTo(Date.from(zonedEndOfWeek.toInstant()));
            dateRanges.add(dateRange);
        }

        return dateRanges;
    }
}
