package com.tofly.xrtygis.custom.controller;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.xrtygis.custom.entity.Lscreen.*;
import com.tofly.xrtygis.custom.service.LscreenNewService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 智慧营销接口
 */
@RestController
@RequestMapping("/lscreenNew")
@Api(tags = "大屏展示_智慧营销接口(新)")
public class LscreenNewController {
  @Autowired
    private LscreenNewService lscreenNewService;
  @Value("${waterSupplyChu.codes}")
  private String waterSupplyChu;


  @GetMapping("/getUserCount")
  @ApiOperation("用户统计")
  public ResultRespone getUserCount(){
    return ResultRespone.success(lscreenNewService.getUserCount());
  }

  @GetMapping("/getUserWaterType")
  @ApiOperation("用户用水性质分类统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "startMonth",value = "开始月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "endMonth",value = "结束月份yyyy-MM",required = true,dataType = "String"),
  })
  public ResultRespone<List<UserWaterTypeVo>> getUserWaterType(String startMonth, String endMonth){
    return ResultRespone.success(lscreenNewService.getUserWaterType(startMonth,endMonth));

  }
  @GetMapping("/getUserWaterTypeMoney")
  @ApiOperation("用户收费数据展示（按照用水性质）")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "startMonth",value = "开始月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "endMonth",value = "结束月份yyyy-MM",required = true,dataType = "String"),
  })
  public ResultRespone<List<UserWaterTypeMoney>> getUserWaterTypeMoney(String startMonth, String endMonth){
    return ResultRespone.success(lscreenNewService.getUserWaterTypeMoney(startMonth,endMonth));
  }
  @GetMapping("/getUserWaterMoney")
  @ApiOperation("年度同期收费数据")
  public ResultRespone<UserWaterMoneyVo> getUserWaterMoney(){
    return ResultRespone.success(lscreenNewService.getUserWaterMoney());
  }

  @GetMapping("/getPayMoneyYear")
  @ApiOperation("今年，去年当前收费数据对比")
  public ResultRespone<UserMoneyInfoVo> getPayMoneyYear(){
    return ResultRespone.success(lscreenNewService.getPayMoneyYear());
  }

  @GetMapping("/getUserWaterAmount")
  @ApiOperation("近13月售水量,金额统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "startMonth",value = "开始月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "endMonth",value = "结束月份yyyy-MM",required = true,dataType = "String"),
  })
  public ResultRespone<List<UserWaterMoney>> getUserWaterAmount(String startMonth, String endMonth){
    return ResultRespone.success(lscreenNewService.getUserWaterMoneyComm(startMonth,endMonth,waterSupplyChu));
  }

  @GetMapping("/getUserStatistics")
  @ApiOperation("近12月用户新增，销户统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "startMonth",value = "开始月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "endMonth",value = "结束月份yyyy-MM",required = true,dataType = "String"),
  })
  public ResultRespone<List<UserWaterType>> getUserStatistics(String startMonth, String endMonth){
    return ResultRespone.success(lscreenNewService.getUserStatistics(startMonth,endMonth));
  }

  @GetMapping("/getMeterReadingMonth")
  @ApiOperation("上月本月抄表率统计")
  public ResultRespone getMeterReadingMonth(){
    return ResultRespone.success(lscreenNewService.getMeterReadingMonth());

  }

  @GetMapping("/getWaterCategoryModel")
  @ApiOperation("水表种类统计以及种类下型号统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "endMonth",value = "截止月份",required = false,dataType = "String"),
  })
  public ResultRespone<List<WaterCategoryVo>> getWaterCategoryModel(String endMonth){
    return ResultRespone.success(lscreenNewService.getWaterCategoryModel(endMonth));
  }

  @GetMapping("/getWaterFactory")
  @ApiOperation("水表厂家下对应水表种类统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "endMonth",value = "截止月份",required = false,dataType = "String"),
  })
  public ResultRespone<List<WaterFactoryVo>> getWaterFactory(String endMonth){
    return ResultRespone.success(lscreenNewService.getWaterFactory(endMonth));
  }

  @GetMapping("/getCostCategory")
  @ApiOperation("收费情况统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "year",value = "年",required = true,dataType = "Integer")
  })
  public ResultRespone getCostCategory(Integer year){
    return ResultRespone.success(lscreenNewService.getCostCategory(year));
  }
  @GetMapping("/getLargeUserWater")
  @ApiOperation("大用户用水/费用/排行统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "year",value = "年",required = true,dataType = "Integer"),
          @ApiImplicitParam(name = "type",value = "类型,1用水排行统计，2费用排行统计",required = true,dataType = "Integer"),
          @ApiImplicitParam(name = "topNum",value = "前多少数据",required = true,dataType = "Integer"),
  })
  public ResultRespone getLargeUserWater(Integer year,Integer type,Integer topNum){
    return ResultRespone.success(lscreenNewService.getLargeUserWater(year,type,topNum));
  }

  @GetMapping("/getLargeUserArrears")
  @ApiOperation("大用户欠费排行统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "startMonth",value = "开始月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "endMonth",value = "结束月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "topNum",value = "前多少数据",required = true,dataType = "Integer"),
  })
  public ResultRespone getLargeUserArrears(String startMonth,String endMonth,Integer topNum){
    return ResultRespone.success(lscreenNewService.getLargeUserArrears(startMonth, endMonth,topNum));
  }

  @GetMapping("/getUserLevel")
  @ApiOperation("用户信用等级")
  public ResultRespone getUserLevel(){
    return ResultRespone.success(lscreenNewService.getUserLevel());
  }

  @GetMapping("/getCollectionRate")
  @ApiOperation("今年去年回款率统计")
  public ResultRespone<CollectionRateVo> getCollectionRateYear(){
    return ResultRespone.success(lscreenNewService.getCollectionRateYear());
  }
  @GetMapping("/getChargeType")
  @ApiOperation("收费方式统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "startMonth",value = "开始月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "endMonth",value = "结束月份yyyy-MM",required = true,dataType = "String"),
  })
  public ResultRespone getChargeType(String startMonth,String endMonth){
    return ResultRespone.success(lscreenNewService.getChargeType(startMonth,endMonth));
  }


  @GetMapping("/getHotlineCategoryInfo")
  @ApiOperation("近12月热线统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "startMonth",value = "开始月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "endMonth",value = "结束月份yyyy-MM",required = true,dataType = "String"),
  })
  public ResultRespone<HotlineCategoryVo> getHotlineCategoryInfo(String startMonth, String endMonth){
    return ResultRespone.success(lscreenNewService.getHotlineCategoryInfo(startMonth,endMonth));
  }

  @GetMapping("/getHotlineMethodInfo")
  @ApiOperation("近12月热线呼入，呼出，回访统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "startMonth",value = "开始月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "endMonth",value = "结束月份yyyy-MM",required = true,dataType = "String"),
  })
  public ResultRespone<List<HotlineMethodInfoVo>> getHotlineMethodInfo(String startMonth, String endMonth){
    return ResultRespone.success(lscreenNewService.getHotlineMethodInfo(startMonth,endMonth));
  }


  @GetMapping("/getHotlineOrderStatusInfo")
  @ApiOperation("热线工单状态统计（年）")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "year",value = "年",required = true,dataType = "Integer")
  })
  public ResultRespone getHotlineOrderStatusInfo(Integer year){
    return ResultRespone.success(lscreenNewService.getHotlineOrderStatusInfo(year));
  }

  @GetMapping("/getHotlineOrderTypeInfo")
  @ApiOperation("热线工单统计(周)")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "weeks",value = "周数",required = true,dataType = "Integer")
  })
  public ResultRespone<List<HotlineOrderTypeVo>> getHotlineOrderTypeInfo(Integer weeks){
    return ResultRespone.success(lscreenNewService.getHotlineOrderTypeInfo(weeks));
  }

  @GetMapping("/getHotlineServiceInfo")
  @ApiOperation("优质服务统计情况（年）")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "year",value = "年",required = true,dataType = "Integer")
  })
  public ResultRespone getHotlineServiceInfo(Integer year){

    return ResultRespone.success(lscreenNewService.getHotlineServiceInfo(year));
  }
  @GetMapping("/getHotlineMessageTypeInfo")
  @ApiOperation("短信类型统计")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "startMonth",value = "开始月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "endMonth",value = "结束月份yyyy-MM",required = true,dataType = "String"),
  })
  public ResultRespone getHotlineMessageTypeInfo(String startMonth, String endMonth){

    return ResultRespone.success(lscreenNewService.getHotlineMessageTypeInfo(startMonth,endMonth));
  }
  @GetMapping("/getHotlineMessageTotal")
  @ApiOperation("短信次数统计按照年月，返回每月")
  @ApiImplicitParams({
          @ApiImplicitParam(name = "startMonth",value = "开始月份yyyy-MM",required = true,dataType = "String"),
          @ApiImplicitParam(name = "endMonth",value = "结束月份yyyy-MM",required = true,dataType = "String")
  })
  public ResultRespone<List<HotlineMessageTypeInfo>> getHotlineMessageTotal(String startMonth, String endMonth){

    return ResultRespone.success(lscreenNewService.getHotlineMessageTotal(startMonth,endMonth));
  }


  @GetMapping("/getHotlineMessageYear")
  @ApiOperation("短信次数统计（今年，去年月对比）")
  public ResultRespone<HotlineMessageCountVo> getHotlineMessageYear(){

    return ResultRespone.success(lscreenNewService.getHotlineMessageYear());
  }
}
