package com.tofly.xrtygis.custom.service.impl;
import com.tofly.common.core.util.StringUtil;
import com.tofly.xrtygis.commons.CalendarHelper;
import com.tofly.xrtygis.commons.DateWeekUtil;
import com.tofly.xrtygis.custom.entity.Lscreen.*;
import com.tofly.xrtygis.custom.mapper.LscreenNewMapper;
import com.tofly.xrtygis.custom.service.LscreenNewService;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.Collator;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class LscreenNewServiceImpl implements LscreenNewService {
    @Autowired
    private LscreenNewMapper lscreenNewMapper;

    @Override
    public Object getUserCount() {
        Integer year=LocalDate.now().getYear();
        Integer lastYear=LocalDate.now().getYear()-1;
        String recordMonth=lscreenNewMapper.getMaxRecordMonth(year);
        return lscreenNewMapper.getUserCount(year,lastYear,recordMonth);

    }

    @Override
    public List<UserWaterTypeVo> getUserWaterType(String startMonth, String endMonth) {
       List<String> dates=CalendarHelper.getMonthBetweenDate(startMonth,endMonth,"yyyy-MM");
        List<UserWaterType>  volist=lscreenNewMapper.getUserWaterType(startMonth,endMonth);
        List<UserWaterTypeVo> listResult=new ArrayList<>();
        Map<String,List<UserWaterType>> map=volist.stream().collect(Collectors.groupingBy(UserWaterType::getWaterCategory));
        map.forEach((waterCategory,listMap)->{
            UserWaterTypeVo vo=new UserWaterTypeVo();
            vo.setWaterCategory(waterCategory);
            List<UserWaterType> waterTypes=new ArrayList<>();
            dates.forEach(date->{
                UserWaterType userWaterType= listMap.stream().filter(item->item.getRecordMonth().equals(date)).findFirst().orElse(null);
                if(userWaterType==null){
                    userWaterType=new UserWaterType();
                    userWaterType.setRecordMonth(date);
                    userWaterType.setWaterCategory(waterCategory);
                }
                waterTypes.add(userWaterType);

            });
           vo.setListWaterType(waterTypes);
            listResult.add(vo);
        });
        return listResult;

    }


    @Override
    public  List<UserWaterTypeMoney> getUserWaterTypeMoney(String startMonth, String endMonth){
        return lscreenNewMapper.getUserWaterTypeMoney(startMonth,endMonth);
    }

    @Override
    public  UserWaterMoneyVo getUserWaterMoney(){
        UserWaterMoneyVo userWaterMoneyVo=new UserWaterMoneyVo();
        Integer nowYear=LocalDate.now().getYear();
        String startMonth=nowYear+"-01";
        String endMonth=nowYear+"-12";
        String lastStartMonth=nowYear-1+"-01";
        String lastEndMonth=nowYear-1+"-12";
        userWaterMoneyVo.setNowYearList(getUserWaterMoneyComm(startMonth,endMonth,""));
        userWaterMoneyVo.setLastYearList(getUserWaterMoneyComm(lastStartMonth,lastEndMonth,""));
        return userWaterMoneyVo;
    }

    @Override
    public  List<UserWaterMoney> getUserWaterMoneyComm(String startMonth,String endMonth,String waterSupplyChu){
      return  lscreenNewMapper.getUserWaterMoney(startMonth,endMonth,waterSupplyChu);

    }



    @Override
    public  List<UserWaterType> getUserStatistics(String startMonth, String endMonth){
        return lscreenNewMapper.getUserStatistics(startMonth,endMonth);
    }

    @Override
    public Object getMeterReadingMonth() {
        Calendar calendar=Calendar.getInstance();
        String  endMonth=CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM");
         calendar.add(Calendar.MONTH, -1);
         String startMonth= CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM");
        List<Map<String,Object>> mapList= lscreenNewMapper.getMeterReadingMonth(startMonth,endMonth);
        Map<String,Object> result=new HashMap<>();

        mapList.forEach(map->{
            if(map.get("recordMonth").equals(startMonth)){
                result.put("lastCopyRate", map.get("copyRate"));
                result.put("lastMeterReadingRate",map.get("meterReadingRate"));
            };
            if(map.get("recordMonth").equals(endMonth)){
                result.put("nowCopyRate",  map.get("copyRate"));
                result.put("nowMeterReadingRate",  map.get("meterReadingRate"));
            };

        });
        return result;
    }

    @Override
    public List<WaterCategoryVo> getWaterCategoryModel(String endMonth) {
        List<WaterCategoryVo> waterCategoryVoList=new ArrayList<>();
        if(StringUtil.isBlank(endMonth)){
            endMonth=getWaterInfoMaxRecordMonth();
        }
       List<WaterCategoryInfo> list=lscreenNewMapper.getWaterFactoryInfo(endMonth,endMonth);

        List<String> waterCategoryList = list.stream()
                .map(WaterCategoryInfo::getWaterCategory) // 假设getField是获取字段值的方法
                .distinct()
                .collect(Collectors.toList());
        //按照中文排序
        Collections.sort(waterCategoryList, (o1, o2) -> {
            Collator ca=Collator.getInstance();
            return  ca.compare(o1,o2);
        });

       // Map<String,List<WaterCategoryInfo>> listMap=  list.stream().collect(Collectors.groupingBy(WaterCategoryInfo::getWaterCategory));

        waterCategoryList.forEach(waterCategory->{
            WaterCategoryVo waterCategoryVo=new WaterCategoryVo();
            waterCategoryVo.setWaterCategory(waterCategory);
            List<WaterCategoryInfo> listWaterCategory=list.stream().filter(item->item.getWaterCategory().equals(waterCategory)).collect(Collectors.toList());
            Integer waterCount=0;
            if(listWaterCategory!=null && listWaterCategory.size()>0){

                waterCount=listWaterCategory.stream().mapToInt(a->a.getWaterCount()).sum();
            }
            waterCategoryVo.setWaterCount(waterCount);

            List<WaterCategoryInfo> listWaterModelVo=new ArrayList<>();

            Map<String, List<WaterCategoryInfo>>mapWaterModel=listWaterCategory.stream().filter(item->StringUtil.isNotBlank(item.getWaterModel())).collect(Collectors.groupingBy(WaterCategoryInfo::getWaterModel));
            if(!mapWaterModel.isEmpty()){
                mapWaterModel.forEach((waterModel,listWaterModel)->{
                    WaterCategoryInfo waterCategoryInfo=new WaterCategoryInfo();
                    waterCategoryInfo.setWaterCategory(waterCategory);
                    waterCategoryInfo.setWaterModel(waterModel);
                    Integer waterModelCount=listWaterModel.stream().mapToInt(item->item.getWaterCount()).sum();
                    waterCategoryInfo.setWaterCount(waterModelCount);
                    listWaterModelVo.add(waterCategoryInfo);
                });
            }
            waterCategoryVo.setListCategoryModel(listWaterModelVo);
            waterCategoryVoList.add(waterCategoryVo);

        });

        return waterCategoryVoList;
    }

    private String getWaterInfoMaxRecordMonth(){
        String tableName="YX_REPORT_WATER_INFO@YXDBLINK";
        return lscreenNewMapper.getMaxRecordMonthByTable(tableName,null);

    }

    @Override
    public List<WaterFactoryVo> getWaterFactory(String endMonth) {
        if(StringUtil.isBlank(endMonth)){
            endMonth=getWaterInfoMaxRecordMonth();
        }
         List<WaterCategoryInfo> list=lscreenNewMapper.getWaterFactoryInfo(endMonth,endMonth);
         List<WaterFactoryVo> waterFactoryVoList=new ArrayList<>();
        List<String> waterCategoryList = list.stream()
                .map(WaterCategoryInfo::getWaterCategory) // 假设getField是获取字段值的方法
                .distinct()
                .collect(Collectors.toList());
        //按照中文排序
        Collections.sort(waterCategoryList, (o1, o2) -> {
            Collator ca=Collator.getInstance();
            return  ca.compare(o1,o2);
        });

        Map<String,List<WaterCategoryInfo>> listMap=list.stream().collect(Collectors.groupingBy(WaterCategoryInfo::getWaterFactory));

        listMap.forEach((waterFactory,mapList)->{
            WaterFactoryVo vo=new WaterFactoryVo();
            vo.setWaterFactory(waterFactory);
            Integer waterTotalCount=mapList.stream().mapToInt(a->a.getWaterCount()).sum();
            vo.setWaterTotalCount(waterTotalCount);
            List<WaterCategoryVo> waterCategoryVos=new ArrayList<>();
             waterCategoryList.forEach(waterCategory->{
                WaterCategoryVo waterCategoryVo=new WaterCategoryVo();
                waterCategoryVo.setWaterCategory(waterCategory);
                List<WaterCategoryInfo> listCategory= mapList.stream().filter(item->item.getWaterCategory().equals(waterCategory)).collect(Collectors.toList());
                Integer waterCount=null;
                if(listCategory!=null && listCategory.size()>0){
                    waterCount=listCategory.stream().mapToInt(a->a.getWaterCount()).sum();
                }
                waterCategoryVo.setWaterCount(waterCount);
                waterCategoryVos.add(waterCategoryVo);
            });

             vo.setListCategory(waterCategoryVos);
            waterFactoryVoList.add(vo);
        });
         return waterFactoryVoList;
    }

    @Override
    public Object getCostCategory(Integer year) {
        return lscreenNewMapper.getCostCategory(year);
    }

    @Override
    public Object getLargeUserWater(Integer year, Integer type, Integer topNum) {
        if(type.equals(1)){
            return lscreenNewMapper.getLargeUserWater(year,topNum);
        }else{
            return lscreenNewMapper.getLargeUserMoney(year,topNum);
        }

    }

    @Override
    public Object getLargeUserArrears(String startMonth, String endMonth, Integer topNum) {
        return lscreenNewMapper.getLargeUserArrears(startMonth,endMonth,topNum);
    }

    @Override
    public Object getUserLevel() {
        return lscreenNewMapper.getUserLevel();
    }

    @Override
    public CollectionRateVo getCollectionRateYear() {
        CollectionRateVo vo=new CollectionRateVo();
        Integer nowYear=LocalDate.now().getYear();
        String startMonth=nowYear+"-01";
        String endMonth=nowYear+"-12";
        String lastStartMonth=nowYear-1+"-01";
        String lastEndMonth=nowYear-1+"-12";
        vo.setNowYearList(lscreenNewMapper.getCollectionRate(startMonth,endMonth));
        vo.setLastYearList(lscreenNewMapper.getCollectionRate(lastStartMonth,lastEndMonth));
        return vo;
    }

    @Override
    public Object getChargeType(String startMonth, String endMonth) {
       return lscreenNewMapper.getChargeType(startMonth,endMonth);
    }

    @Override
    public HotlineCategoryVo getHotlineCategoryInfo(String startMonth, String endMonth) {
        HotlineCategoryVo hotlineCategoryVo=new HotlineCategoryVo();
        List<HotlineCategoryInfo> list=lscreenNewMapper.getHotlineCategoryInfo(startMonth,endMonth);

        List<String> channelTypeList = list.stream()
                .map(HotlineCategoryInfo::getChannelType) // 假设getField是获取字段值的方法
                .distinct()
                .collect(Collectors.toList());
        //按照中文排序
        Collections.sort(channelTypeList, (o1, o2) -> {
            Collator ca=Collator.getInstance();
            return  ca.compare(o1,o2);
        });





        Map<String,List<HotlineCategoryInfo>> mapCategory=list.stream().collect(Collectors.groupingBy(HotlineCategoryInfo::getCategoryName));
        List<HotlineCategoryType> hotlineCategoryVos=new ArrayList<>();

        mapCategory.forEach((categoryName,listCategory)->{
            HotlineCategoryType vo=new HotlineCategoryType();
            vo.setCategoryName(categoryName);
            Integer totalCount=listCategory.stream().mapToInt(a->a.getChannelTypeCount()).sum();
            vo.setTotalCount(totalCount);
            List<HotlineCategoryInfo> hotlineCategoryInfos=new ArrayList<>();
            channelTypeList.forEach(channelType->{
                HotlineCategoryInfo hotlineCategoryInfo=listCategory.stream().filter(item->item.getChannelType().equals(channelType)).findFirst().orElse(null);
              if(hotlineCategoryInfo==null){
                  hotlineCategoryInfo=new HotlineCategoryInfo();
                  hotlineCategoryInfo.setChannelType(channelType);
                  hotlineCategoryInfo.setCategoryName(categoryName);
                  hotlineCategoryInfo.setChannelTypeCount(0);

              }
                hotlineCategoryInfos.add(hotlineCategoryInfo);
            });
            vo.setListCategory(hotlineCategoryInfos);
            hotlineCategoryVos.add(vo);
        });

        List<HotlineChannelType> hotlineChannelTypeVos=new ArrayList<>();
        channelTypeList.forEach(channelType->{
                HotlineChannelType vo=new HotlineChannelType();
                vo.setChannelType(channelType);
                List<HotlineCategoryInfo> hotlineCategoryInfos=list.stream().filter(item->item.getChannelType().equals(channelType)).collect(Collectors.toList());
                Integer totalCount=0;
                if(hotlineCategoryInfos!=null && hotlineCategoryInfos.size()>0){
                     totalCount= hotlineCategoryInfos.stream().mapToInt(a->a.getChannelTypeCount()).sum();
                }
                vo.setTotalCount(totalCount);
                hotlineChannelTypeVos.add(vo);
           });


        hotlineCategoryVo.setListCategory(hotlineCategoryVos);
        hotlineCategoryVo.setListChannelType(hotlineChannelTypeVos);
        return hotlineCategoryVo;
    }

    @Override
    public  List<HotlineMethodInfoVo> getHotlineMethodInfo(String startMonth, String endMonth){
     List<HotlineMethodInfoVo> hotlineMethodInfoVos=new ArrayList<>();
        List<HotlineMethodInfo> hotlineMethodInfos= lscreenNewMapper.getHotlineMethodInfo(startMonth,endMonth);
        List<String> dates=CalendarHelper.getMonthBetweenDate(startMonth,endMonth,"yyyy-MM");
       Map<String,List<HotlineMethodInfo>> mapList=hotlineMethodInfos.stream().collect(Collectors.groupingBy(HotlineMethodInfo::getCategoryName));
        mapList.forEach((categoryName,listCategoryName)->{
            Integer totalCount=listCategoryName.stream().mapToInt(HotlineMethodInfo::getCategoryCount).sum();
            HotlineMethodInfoVo hotlineMethodInfoVo=new HotlineMethodInfoVo();
            hotlineMethodInfoVo.setCategoryName(categoryName);
            hotlineMethodInfoVo.setTotalCount(totalCount);
            List<HotlineMethodInfo> listInfo=new ArrayList<>();
            dates.forEach(date->{
                HotlineMethodInfo hotlineMethodInfo= listCategoryName.stream().filter(item->item.getRecordMonth().equals(date)).findFirst().orElse(null);
                   if(hotlineMethodInfo==null){
                       hotlineMethodInfo=new HotlineMethodInfo();
                       hotlineMethodInfo.setRecordMonth(date);
                       hotlineMethodInfo.setCategoryCount(0);
                       hotlineMethodInfo.setCategoryName(categoryName);
                   }
                listInfo.add(hotlineMethodInfo);
            });
            hotlineMethodInfoVo.setListInfo(listInfo);
            hotlineMethodInfoVos.add(hotlineMethodInfoVo);
        });
            return hotlineMethodInfoVos;

    }

    @Override
    public Object getHotlineOrderStatusInfo(Integer year) {
        return lscreenNewMapper.getHotlineOrderStatusInfo(year);
    }
    @Override
    public Object getHotlineServiceInfo(Integer year) {
        return lscreenNewMapper.getHotlineServiceInfo(year);
    }

    @Override
    public Object getHotlineMessageTypeInfo(String startMonth, String endMonth) {
        return lscreenNewMapper.getHotlineMessageTypeInfo(startMonth,endMonth);
    }

    @Override
    public List<HotlineMessageTypeInfo> getHotlineMessageTotal(String startMonth, String endMonth) {
        return lscreenNewMapper.getHotlineMessageTotal(startMonth,endMonth);
    }
    @Override
    public  HotlineMessageCountVo getHotlineMessageYear() {
        HotlineMessageCountVo vo=new HotlineMessageCountVo();
        Integer nowYear=LocalDate.now().getYear();
        String startMonth=nowYear+"-01";
        String endMonth=nowYear+"-12";
        String lastStartMonth=nowYear-1+"-01";
        String lastEndMonth=nowYear-1+"-12";
        vo.setNowYearList(lscreenNewMapper.getHotlineMessageTotal(startMonth,endMonth));
        vo.setLastYearList(lscreenNewMapper.getHotlineMessageTotal(lastStartMonth,lastEndMonth));
        return vo;

    }


    @Override
    public List<HotlineOrderTypeVo> getHotlineOrderTypeInfo(Integer weeks) {
        int days=6;
        List<HotlineOrderTypeVo> hotlineOrderTypeVos=new ArrayList<>();
        Date nowTime=new Date();
        String simpleDateFormat="yyyy-MM-dd";
       List<Date> listDates=DateWeekUtil.getFirstWeeksDate(weeks,nowTime);
        Collections.sort(listDates); //排序，需要验证
        Date endDate = getEndDate(Collections.max(listDates),days);
        Date startDate = Collections.min(listDates);

        List<HotlineOrderTypeInfo> hotlineOrderTypeInfos=lscreenNewMapper.getHotlineOrderTypeInfo(startDate,endDate);
       Map<String,List<HotlineOrderTypeInfo>> listMap=hotlineOrderTypeInfos.stream().collect(Collectors.groupingBy(HotlineOrderTypeInfo::getOrderType));
        listMap.forEach((orderType,listOrderType)->{
            List<HotlineOrderWeek> hotlineOrderWeeks=new ArrayList<>();
            listDates.forEach(weekFirstDay->{
                Date weekLastDay=getEndDate(weekFirstDay,days);
                String weekStartDate=CalendarHelper.getFormatDate(weekFirstDay,simpleDateFormat);
                String weekEndDate= CalendarHelper.getFormatDate(weekLastDay,simpleDateFormat);
                LocalDateTime localDateTime=DateWeekUtil.convertToLocalDateTime(weekFirstDay);
                HotlineOrderWeek hotlineOrderWeek=new HotlineOrderWeek();
                hotlineOrderWeek.setOrderType(orderType);
                hotlineOrderWeek.setWeekName(DateWeekUtil.getMonthNoAndWeekNo(localDateTime));
                hotlineOrderWeek.setWeekStartDate(weekStartDate);
                hotlineOrderWeek.setWeekEndDate(weekEndDate);
                Integer orderCount=0;
              List<HotlineOrderTypeInfo> weekLists=listOrderType.stream().
                      filter(item->item.getRecordDay().getTime()>=weekFirstDay.getTime() &&
                        item.getRecordDay().getTime()<=weekLastDay.getTime()).collect(Collectors.toList());
              if(weekLists!=null && weekLists.size()>0){
                  orderCount=weekLists.stream().mapToInt(HotlineOrderTypeInfo::getOrderCount).sum();
              }

                hotlineOrderWeek.setOrderCount(orderCount);
                hotlineOrderWeeks.add(hotlineOrderWeek);
            });
            Integer orderCount=listOrderType.stream().mapToInt(HotlineOrderTypeInfo::getOrderCount).sum();
            HotlineOrderTypeVo vo=new HotlineOrderTypeVo();
            vo.setOrderType(orderType);
            vo.setOrderCount(orderCount);
            vo.setOrderWeekList(hotlineOrderWeeks);
            hotlineOrderTypeVos.add(vo);
        });

        return hotlineOrderTypeVos;
    }

    @Override
    public UserMoneyInfoVo getPayMoneyYear() {
        Calendar calendar=Calendar.getInstance();
        calendar.add(Calendar.MONTH, -1);
        Integer nowYear=calendar.get(Calendar.YEAR);
        //今年截止月份，结束到上个月
        String nowYearMonthEnd= CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM") ;
        calendar.add(Calendar.YEAR, -1);
        //去年截止月份
        String lastYearMonthEnd= CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM") ;
        Integer lastYear=nowYear-1;
        //得到今年和去年数据最大的月份，用户总数只需要取最大月份的数据即可
        String nowYearMonth=lscreenNewMapper.getMaxRecordMonth(nowYear);
        String lastYearMonth=lscreenNewMapper.getMaxRecordMonth(lastYear);
        String nowYearMonthStart=nowYear+"-01"; //今年开始月份
        String lastYearMonthStart=lastYear+"-01"; //去年开始月份
        return lscreenNewMapper.getUserCountByMonth(nowYearMonth,lastYearMonth,nowYearMonthStart,nowYearMonthEnd,lastYearMonthStart,lastYearMonthEnd);

    }

    private Date  getEndDate(Date date,int days){
        Calendar calendar=Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, days);
       return calendar.getTime();
    }
}
