package com.tofly.custom.controller;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.haiqiu.tools.common.utils.HttpServletUtils;
import com.tofly.common.ExcelTool.ExcelColumn;
import com.tofly.common.ExcelTool.ExcelTool;
import com.tofly.common.constant.CalendarHelper;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.custom.entity.dto.ScadaFlowReportDto;
import com.tofly.custom.entity.dto.ScadaReportDto;
import com.tofly.custom.entity.dto.WisdomDisplayDto;
import com.tofly.custom.entity.vo.*;
import com.tofly.custom.service.CustomDisplayService;
import com.tofly.scada.entity.DeviceArchiveManage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author JF
 * @date 2023-1-6
 * @description 自定义智慧综合展示相关
 */
@RestController
@RequestMapping("/customDisplay")
/*@AllArgsConstructor*/
@RefreshScope
@Api(tags = "自定义智慧综合展示")
public class CustomDisplayController {

    @Autowired
    private CustomDisplayService customDisplayService;

    /**
     * 水厂进指标
     */
    @Value("${waterSupplyJin.codesAll}")
    private String waterSupplyJinAll;
    /**
     * 水厂出指标
     */
    @Value("${waterSupplyChu.codesAll}")
    private String waterSupplyChuAll;

    /**
     * 水厂总瞬时流量
     */
    @Value("${waterSupplyChu.codesFlow}")
    private String waterSupplyChuFlow;


    /**
     * 水厂进指标
     */
    @Value("${waterSupplyJin.codes}")
    private String waterSupplyJin;


    /**
     * 水厂出指标
     */
    @Value("${waterSupplyChu.codes}")
    private String waterSupplyChu;


    @Value("${waterSupplyJin.codesWs}")
    private String waterSupplyJinWs;
    /**
     * 污水厂出指标
     */
    @Value("${waterSupplyChu.codesWs}")
    private String waterSupplyChuWs;


    /**
     * @param wisdomDisplayDto 自定义智慧综合展示筛选字段对象
     */
    @PostMapping("/viewStationDeviceIndex")
    @ApiOperation(value = "查询厂站设备指标数据（厂站分组）")
    public ResultRespone<List<WaterStationVo>> viewStationDeviceIndex(WisdomDisplayDto wisdomDisplayDto) {
        try {

            return ResultRespone.success(customDisplayService.viewStationDeviceIndex(wisdomDisplayDto,waterSupplyJinAll,waterSupplyChuAll));
        } catch (Exception e) {
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    /**
     * @param wisdomDisplayDto 自定义智慧综合展示筛选字段对象
     */
    @PostMapping("/viewStationIndex")
    @ApiOperation(value = "查询厂站指标数据（厂站分组）")
    public ResultRespone<List<WaterStationVo>> viewStationIndex(WisdomDisplayDto wisdomDisplayDto) {
        try {
            return ResultRespone.success(customDisplayService.viewStationIndex(wisdomDisplayDto,waterSupplyJinAll,waterSupplyChuAll));
        } catch (Exception e) {
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    /**
     * @param wisdomDisplayDto 自定义智慧综合展示筛选字段对象
     */
    @PostMapping("/viewDeviceIndex")
    @ApiOperation(value = "查询设备指标数据（设备分组）")
    public ResultRespone<List<DeviceArchiveManage>> viewDeviceIndex(WisdomDisplayDto wisdomDisplayDto) {
        try {
            return ResultRespone.success(customDisplayService.viewDeviceIndex(wisdomDisplayDto));
        } catch (Exception e) {
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    /**
     * @param startDate  开始时间（yyyy-MM-dd HH:mm:ss）
     * @param endDate    结束时间（yyyy-MM-dd HH:mm:ss）
     * @param indexTypes 指标类型（多个用,分隔）
     */
    @GetMapping("/getWaterSourceScada")
    @ApiOperation(value = "统计不同水源的SCADA数据")
    public ResultRespone<List<WaterSourceScadaVo>> getWaterSourceScada(@ApiParam("开始时间（yyyy-MM-dd HH:mm:ss）") @RequestParam(value = "startDate") String startDate,
                                                                       @ApiParam("结束时间（yyyy-MM-dd HH:mm:ss）") @RequestParam(value = "endDate") String endDate,
                                                                       @ApiParam("指标类型（多个用,分隔）") @RequestParam(value = "indexTypes", required = false) String indexTypes) {
        try {
            return ResultRespone.success(customDisplayService.getWaterSourceScada(startDate, endDate, indexTypes));
        } catch (Exception e) {
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    /**
     * @param
     * @description 指标相关值统计
     */
    @GetMapping("/indexValCensus")
    @ApiOperation(value = "指标相关值统计")
    public ResultRespone<IndexValCensusVo> indexValCensus() {
        try {
            return ResultRespone.success(customDisplayService.indexValCensus());
        } catch (Exception e) {
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @PostMapping("/getScadaIndexData")
    @ApiOperation(value = "查询SCADA指标数据")

    public ResultRespone<List<ScadaIndexVo>> getScadaIndexData(WisdomDisplayDto dto){
        try {
            return ResultRespone.success(customDisplayService.getScadaIndexData(dto));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @PostMapping("/getScadaHistoryData")
    @ApiOperation(value = "查询SCADA历史数据")

    public ResultRespone getHistoryPageData(Page page,WisdomDisplayDto dto){
        try {
            return ResultRespone.success(customDisplayService.getHistoryPageData(page,dto));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/getUserTotalMonth")
    @ApiOperation(value = "按月统计用户总数")

    public ResultRespone<List<UserTotalVo>> getUserTotalMonth(@ApiParam("开始时间（yyyy-MM-dd HH:mm:ss）") @RequestParam(value = "startDate") String startDate,
                                                               @ApiParam("结束时间（yyyy-MM-dd HH:mm:ss）") @RequestParam(value = "endDate") String endDate){
        try {
            return ResultRespone.success(customDisplayService.getUserTotalMonth(startDate, endDate));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/getDeviceByType")
    @ApiOperation(value = "通过设备类型分组获取设备列表")

    public ResultRespone<List<DeviceByTypeVo>> getDeviceByType(@ApiParam("设备名称（模糊查询）") @RequestParam(value = "deviceName", required = false) String deviceName){
        try {
            return ResultRespone.success(customDisplayService.getDeviceByType(deviceName));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }


    @GetMapping("/getIndexListByType")
    @ApiOperation(value = "根据指标类型查询指标（水厂）")
    public ResultRespone getIndexListByType(
            @ApiParam("水厂类型，支持多个") @RequestParam(value = "waterType", required = false)
          String waterType,
        @ApiParam("指标类型，支持多个") @RequestParam(value = "indexType", required = false)String indexType){

        try {
            return ResultRespone.success(customDisplayService.getIndexListByType(waterType,indexType));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/getPipeIndexListByType")
    @ApiOperation(value = "根据指标类型查询指标（管网）")
    public ResultRespone getPipeIndexListByType(
            @ApiParam("指标类型，多个逗号分离") @RequestParam(value = "indexTypes", required = false)
            String indexTypes,
            @ApiParam("指标编码，多个逗号分离") @RequestParam(value = "codes", required = false)String codes){

        try {
            return ResultRespone.success(customDisplayService.getPipeIndexListByType(indexTypes,codes));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }



    @PostMapping("/getScadaCommReport")
    @ApiOperation(value = "查询SCADA指标数据")

    public ResultRespone<List<ScadaIndexReportVo>> getScadaIndexReport(
           @Validated @RequestBody ScadaReportDto dto){
        try {
            List<ScadaIndexReportVo>  list=customDisplayService.getScadaCommReportNew(waterSupplyChuFlow,dto);
            if(dto.getExportStatus()!=null &&dto.getExportStatus()==1){
                 exportScadaIndexReport(list,dto.getExportTitle(),dto.getExportTotalStatus());
                return ResultRespone.success(null);
            }else{
               return ResultRespone.success(list);
            }
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @PostMapping("/getScadaPipeCommReport")
    @ApiOperation(value = "查询SCADA指标数据(管网)")

    public ResultRespone<List<ScadaIndexReportVo>> getScadaPipeCommReport(
            @Validated @RequestBody ScadaReportDto dto){
        try {
            List<ScadaIndexReportVo>  list=customDisplayService.getScadaPipeCommReport(dto);
            if(dto.getExportStatus()!=null &&dto.getExportStatus()==1){
                exportScadaPipeIndexReport(list,dto.getExportTitle());
                return ResultRespone.success(null);
            }else{
                return ResultRespone.success(list);
            }
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @PostMapping("/getScadaStaticAccFlowReport")
    @ApiOperation(value = "水厂供水出水的总量和污水厂进水总量报表分析")
   public ResultRespone<List<ScadaStaticAccFlowVo>> getScadaStaticReport(
            @Validated @RequestBody ScadaFlowReportDto dto){
        try {
            if(StringUtil.isBlank(dto.getWaterCodes())){
                dto.setWaterCodes(waterSupplyChu);
            }
            if(StringUtil.isBlank(dto.getWaterWsCodes())){
                dto.setWaterWsCodes(waterSupplyJinWs);
            }

            List<ScadaStaticAccFlowVo>  list=customDisplayService.getScadaStaticAccFlowReport(dto);
            if(dto.getExportStatus()!=null && dto.getExportStatus()==1){
                exportScadaStaticReport(list,dto.getExportTitle());
                 return ResultRespone.success(null);

            }else{
                return ResultRespone.success(list);
            }
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    /**
     * 水厂供水出水的总量和污水厂进水总量报表分析（导出）
     * @param list
     * @param exportTitle
     * @throws Exception
     */
    public  void exportScadaStaticReport(List<ScadaStaticAccFlowVo> list,String exportTitle)  throws Exception {

        //构建导出的表头，树形结构，包含标题
        List<Map<String,String>> titleList=new ArrayList<>();
        Map<String,String> titleMap= new HashMap<>();
        //表头标题
        if(StringUtils.isBlank(exportTitle)){
            exportTitle="水厂供水出水的总量和两个污水厂进水瞬时总量报表分析";
        }

        titleMap.put("id","-1");titleMap.put("pid","0");titleMap.put("content",exportTitle);titleMap.put("fielName",null);
        titleList.add(titleMap);
        Map<String,String> titleMap1= new HashMap<>();
        //多表头第一行固定，时间
        titleMap1.put("id","dateTime");titleMap1.put("pid","-1");titleMap1.put("content","时间");titleMap1.put("fielName","dateTime");
        titleList.add(titleMap1);
        Map<String,String> titleMap2= new HashMap<>();
        titleMap2.put("id","waterAmount");titleMap2.put("pid","-1");titleMap2.put("content","水厂出水总量(m³)");titleMap2.put("fielName","waterAmount");
        titleList.add(titleMap2);

        Map<String,String> titleMap3= new HashMap<>();
        titleMap3.put("id","waterWsAmount");titleMap3.put("pid","-1");titleMap3.put("content","污水厂进水总量(m³)");titleMap3.put("fielName","waterWsAmount");
        titleList.add(titleMap3);

        List<Map<String,String>> rowList=new ArrayList<>();
        //循环填充数据
        list.forEach(item->{
            Map<String,String> rowMap= new HashMap<>();
            rowMap.put("dateTime",item.getDateTime());
            rowMap.put("waterAmount",item.getWaterAmount());
            rowMap.put("waterWsAmount",item.getWaterWsAmount());
            rowList.add(rowMap);

        });

        Map<String,String> rowMinTotalMap=new HashMap<String,String>();
        Map<String,String> rowMaxTotalMap=new HashMap<String,String>();
        rowMinTotalMap.put("dateTime","最小值");
        rowMaxTotalMap.put("dateTime","最大值");

        //取总流量最小值，最大值和对应的时间
        Comparator<ScadaStaticAccFlowVo> waterAmountComparator=Comparator.comparing(scada->Double.parseDouble(scada.getWaterAmount()));
        ScadaStaticAccFlowVo waterAmountMin=list.stream().filter(item->item.getWaterAmount()!=null).min(waterAmountComparator).orElse(null);
        ScadaStaticAccFlowVo waterAmountMax=list.stream().filter(item->item.getWaterAmount()!=null).max(waterAmountComparator).orElse(null);

        if(waterAmountMin!=null){
            rowMinTotalMap.put("waterAmount",waterAmountMin.getWaterAmount());
        }
        if(waterAmountMax!=null){
            rowMaxTotalMap.put("waterAmount",waterAmountMax.getWaterAmount());
        }

        Comparator<ScadaStaticAccFlowVo> waterWsAmountComparator=Comparator.comparing(scada->Double.parseDouble(scada.getWaterWsAmount()));
        ScadaStaticAccFlowVo waterWsAmountMin=list.stream().filter(item->item.getWaterWsAmount()!=null).min(waterWsAmountComparator).orElse(null);
        ScadaStaticAccFlowVo waterWsAmountMax=list.stream().filter(item->item.getWaterWsAmount()!=null).max(waterWsAmountComparator).orElse(null);

        if(waterWsAmountMin!=null){
            rowMinTotalMap.put("waterWsAmount",waterWsAmountMin.getWaterWsAmount());
        }
        if(waterWsAmountMax!=null){
            rowMaxTotalMap.put("waterWsAmount",waterAmountMax.getWaterWsAmount());
        }

        rowList.add(rowMinTotalMap);

        rowList.add(rowMaxTotalMap);

        ExcelTool excelTool = new ExcelTool(exportTitle);
        List<ExcelColumn>  titleData=excelTool.columnTransformer(titleList,"id","pid","content","fielName","0");

        excelTool.export(exportTitle, excelTool.exportWorkbook(titleData,rowList,true,true), HttpServletUtils.getServletResponse());

        // ExcelUtils.getListToExcel(ScadaStaticFlowVo.class, list, "水厂供水出水的瞬时流量和两个污水厂进水瞬时流量报表分析");

    }

    @PostMapping("/getScadaStaticFlowReport")
    @ApiOperation(value = "水厂供水出水的瞬时流量和两个污水厂进水瞬时流量报表分析")
    public ResultRespone<List<ScadaStaticFlowVo>> getScadaStaticFlowReport(
            @Validated @RequestBody ScadaFlowReportDto dto){
        try {
            List<ScadaStaticFlowVo>  list=customDisplayService.getScadaStaticFlowReport(dto);
            if(dto.getExportStatus()!=null &&dto.getExportStatus()==1){
                exportScadaStaticFlowReport(list, dto.getExportTitle());
               return ResultRespone.success(null);
            }else{
                return ResultRespone.success(list);
            }
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }
    /**
     * 水厂供水出水的瞬时流量和两个污水厂进水瞬时流量报表分析（导出）
     * @param list
     * @param exportTitle
     * @throws Exception
     */
    public  void exportScadaStaticFlowReport(List<ScadaStaticFlowVo> list,String exportTitle)  throws Exception {

        //构建导出的表头，树形结构，包含标题
        List<Map<String,String>> titleList=new ArrayList<>();
        Map<String,String> titleMap= new HashMap<>();
        //表头标题
        if(StringUtils.isBlank(exportTitle)){
            exportTitle="水厂供水出水的瞬时流量和两个污水厂进水瞬时流量报表分析";
        }

        titleMap.put("id","-1");titleMap.put("pid","0");titleMap.put("content",exportTitle);titleMap.put("fielName",null);
        titleList.add(titleMap);
        Map<String,String> titleMap1= new HashMap<>();
        //多表头第一行固定，时间
        titleMap1.put("id","dateTime");titleMap1.put("pid","-1");titleMap1.put("content","时间");titleMap1.put("fielName","dateTime");
        titleList.add(titleMap1);
        Map<String,String> titleMap2= new HashMap<>();
        titleMap2.put("id","waterAmount");titleMap2.put("pid","-1");titleMap2.put("content","水厂出水总瞬时流量(m³/h)");titleMap2.put("fielName","waterAmount");
        titleList.add(titleMap2);

        Map<String,String> titleMap3= new HashMap<>();
        titleMap3.put("id","waterWsAmount");titleMap3.put("pid","-1");titleMap3.put("content","污水厂进水总瞬时流量(m³/h)");titleMap3.put("fielName","waterWsAmount");
        titleList.add(titleMap3);

        List<Map<String,String>> rowList=new ArrayList<>();
        //列表里面的数据放到listVo里面，方便找最大值，最小值所对应的时间

        //循环填充数据
        list.forEach(item->{
            Map<String,String> rowMap= new HashMap<>();
            rowMap.put("dateTime",item.getDateTime());
            rowMap.put("waterAmount",item.getWaterAmount());
            rowMap.put("waterWsAmount",item.getWaterWsAmount());
            rowList.add(rowMap);

        });
        Map<String,String> rowMinTotalMap=new HashMap<String,String>();
        Map<String,String> rowMaxTotalMap=new HashMap<String,String>();
        rowMinTotalMap.put("dateTime","最小值");
        rowMaxTotalMap.put("dateTime","最大值");

        //取总流量最小值，最大值和对应的时间
        Comparator<ScadaStaticFlowVo> waterAmountComparator=Comparator.comparing(scada->Double.parseDouble(scada.getWaterAmount()));
        ScadaStaticFlowVo waterAmountMin=list.stream().filter(item->StringUtil.isNotBlank(item.getWaterAmount())).min(waterAmountComparator).orElse(null);
        ScadaStaticFlowVo waterAmountMax=list.stream().filter(item->StringUtil.isNotBlank(item.getWaterAmount())).max(waterAmountComparator).orElse(null);

        if(waterAmountMin!=null){
            rowMinTotalMap.put("waterAmount",waterAmountMin.getWaterAmount());
        }
        if(waterAmountMax!=null){
            rowMaxTotalMap.put("waterAmount",waterAmountMax.getWaterAmount());
        }

        Comparator<ScadaStaticFlowVo> waterWsAmountComparator=Comparator.comparing(scada->Double.parseDouble(scada.getWaterWsAmount()));
        ScadaStaticFlowVo waterWsAmountMin=list.stream().filter(item->StringUtil.isNotBlank(item.getWaterWsAmount())).min(waterWsAmountComparator).orElse(null);
        ScadaStaticFlowVo waterWsAmountMax=list.stream().filter(item->StringUtil.isNotBlank(item.getWaterWsAmount())).max(waterWsAmountComparator).orElse(null);

        if(waterWsAmountMin!=null){
            rowMinTotalMap.put("waterWsAmount",waterWsAmountMin.getWaterWsAmount());
        }
        if(waterWsAmountMax!=null){
            rowMaxTotalMap.put("waterWsAmount",waterAmountMax.getWaterWsAmount());
        }

        rowList.add(rowMinTotalMap);

        rowList.add(rowMaxTotalMap);
        ExcelTool excelTool = new ExcelTool(exportTitle);
        List<ExcelColumn>  titleData=excelTool.columnTransformer(titleList,"id","pid","content","fielName","0");
        excelTool.export(exportTitle, excelTool.exportWorkbook(titleData,rowList,true,true), HttpServletUtils.getServletResponse());
        //  excelTool.exportExcel(titleData,rowList,"D://Demo//test6.xlsx",true,true);
    }

    /**
     * 构建树形结构导出
     * @param list 数据集合
     * @param exportTitle 导出标题
     * @throws Exception
     */
    public  void exportScadaIndexReport(List<ScadaIndexReportVo> list,String exportTitle,Integer exportTotalStatus) throws Exception {
        List<Map<String,String>> titleList=new ArrayList<>();
         boolean flag=exportTotalStatus==1?true:false;
        //构建导出的表头，树形结构，包含标题
        Map<String,String> titleMap= new HashMap<>();
        //表头标题
        titleMap.put("id","-1");titleMap.put("pid","0");titleMap.put("content",exportTitle);titleMap.put("fielName",null);
        titleList.add(titleMap);
        Map<String,String> titleMap1= new HashMap<>();
        //多表头第一行固定，时间
        titleMap1.put("id","time");titleMap1.put("pid","-1");titleMap1.put("content","时间");titleMap1.put("fielName","time");
        titleList.add(titleMap1);

        Map<String,String> titleMap2=new HashMap<String,String>();
        //多表头第二行固定，总流量(m³)
        titleMap2.put("id","totalFlow");titleMap2.put("pid","-1");
        titleMap2.put("content","供水总瞬时流量(m³)");titleMap2.put("fielName","totalFlow");

        Map<String,String> titleMap3=new HashMap<String,String>();
        titleMap3.put("id","totalFlowJin");titleMap3.put("pid","-1");
        titleMap3.put("content","取水总瞬时流量(m³)");titleMap3.put("fielName","totalFlowJin");
        if(flag){
            titleList.add(titleMap2);
            titleList.add(titleMap3);
        }

        //取list第一个数据构建动态表头
        List<WaterNameVo> waterNameVos=list.get(0).getWaterNameVos();
        waterNameVos.forEach(waterName->{

             Map<String,String> titleMapWater= new HashMap<>();
             //一级表头，水厂
             titleMapWater.put("id",waterName.getWaterId());
             titleMapWater.put("pid","-1");
             titleMapWater.put("content",waterName.getWaterName());
             titleMapWater.put("fielName",null);
             titleList.add(titleMapWater);
            //一级表头，水厂下二级表头，根据指标来构建
             waterName.getScadaList().forEach(scada->{
                 Map<String,String> scadaMapWater= new HashMap<>();
                 scadaMapWater.put("id",scada.getCode());scadaMapWater.put("pid",scada.getWaterId());
                 if(StringUtils.isNotBlank(scada.getUnit()) && !scada.getUnit().equals("null")){
                     scadaMapWater.put("content",scada.getVariableName()+"("+scada.getUnit()+")");
                 }else{
                     scadaMapWater.put("content",scada.getVariableName());

                 }
                scadaMapWater.put("fielName",scada.getCode());
                 //fielName很重要，填充数据时候，根据fileName对应的值来的
                 titleList.add(scadaMapWater);

         });




       });

        //填充行数据 rowList
        List<Map<String,String>> rowList=new ArrayList<>();
        //列表里面的数据放到listVo里面，方便找最大值，最小值所对应的时间
        List<ScadaFlowVo> listVo=new ArrayList<>();

        String simpleDateFormat="yyyy-MM-dd HH:mm:ss";
        //循环填充数据
        list.forEach(item->{
            Map<String,String> rowMap= new HashMap<>();
            rowMap.put("time",CalendarHelper.getFormatDate(item.getDateTime(),simpleDateFormat));
            if(flag) {
                rowMap.put("totalFlow", getStrDate(item.getTotalFlow(), item.getDateTime(), simpleDateFormat));
                rowMap.put("totalFlowJin", getStrDate(item.getTotalFlowJin(), item.getDateTime(), simpleDateFormat));
            }
            item.getWaterNameVos().forEach(waterNameVo->{
                 waterNameVo.getScadaList().forEach(scadaFlowVo -> {
                     rowMap.put(scadaFlowVo.getCode(),getStrDate(scadaFlowVo.getValue(),scadaFlowVo.getScadaTime(),simpleDateFormat));
                     listVo.add(scadaFlowVo);

                 });

             });
            rowList.add(rowMap);

        });

        Map<String,String> rowMinTotalMap=new HashMap<String,String>();
        Map<String,String> rowMaxTotalMap=new HashMap<String,String>();
        rowMinTotalMap.put("time","最小值");
        rowMaxTotalMap.put("time","最大值");
        if(flag){
        //取总流量最小值，最大值和对应的时间
            Comparator<ScadaIndexReportVo> scadaTotalFlowVoComparator=Comparator.comparing(scada->Double.parseDouble(scada.getTotalFlow()));
            ScadaIndexReportVo totalFlowMin=list.stream().filter(item->StringUtil.isNotBlank(item.getTotalFlow())).min(scadaTotalFlowVoComparator).orElse(null);
            ScadaIndexReportVo totalFlowMax=list.stream().filter(item->StringUtil.isNotBlank(item.getTotalFlow())).max(scadaTotalFlowVoComparator).orElse(null);

            if(totalFlowMin!=null){
                rowMinTotalMap.put("totalFlow",getStrDate(totalFlowMin.getTotalFlow(),totalFlowMin.getDateTime(),simpleDateFormat));

            }
            if(totalFlowMax!=null){
                rowMaxTotalMap.put("totalFlow",getStrDate(totalFlowMax.getTotalFlow(),totalFlowMax.getDateTime(),simpleDateFormat));
            }

            Comparator<ScadaIndexReportVo> scadaTotalFlowJinVoComparator=Comparator.comparing(scada->Double.parseDouble(scada.getTotalFlowJin()));
            ScadaIndexReportVo totalFlowJinMin=list.stream().filter(item->StringUtil.isNotBlank(item.getTotalFlowJin())).min(scadaTotalFlowJinVoComparator).orElse(null);
            ScadaIndexReportVo totalFlowJinMax=list.stream().filter(item->StringUtil.isNotBlank(item.getTotalFlowJin())).max(scadaTotalFlowJinVoComparator).orElse(null);

            if(totalFlowJinMin!=null){
                rowMinTotalMap.put("totalFlowJin",getStrDate(totalFlowJinMin.getTotalFlowJin(),totalFlowJinMin.getDateTime(),simpleDateFormat));

            }
            if(totalFlowJinMax!=null){
                rowMaxTotalMap.put("totalFlowJin",getStrDate(totalFlowJinMax.getTotalFlowJin(),totalFlowJinMax.getDateTime(),simpleDateFormat));
            }


        }
       Map<String,List<ScadaFlowVo>> listMaps=listVo.stream().collect(Collectors.groupingBy(ScadaFlowVo::getCode));
       //按照指标编码分组，取里面的最大值和最小值和对应的时间
          listMaps.forEach((code,voList)->{
                Comparator<ScadaFlowVo> scadaFlowVoComparator=Comparator.comparing(scada->Double.parseDouble(scada.getValue()));
                ScadaFlowVo maxScadaFlowVo=voList.stream().filter(item->StringUtil.isNotBlank(item.getValue())).max(scadaFlowVoComparator).orElse(null);
                ScadaFlowVo minScadaFlowVo=voList.stream().filter(item->StringUtil.isNotBlank(item.getValue())).min(scadaFlowVoComparator).orElse(null);
               if(minScadaFlowVo!=null){
                   rowMinTotalMap.put(code,getStrDate(minScadaFlowVo.getValue(),minScadaFlowVo.getScadaTime(),simpleDateFormat));
               }
                if(maxScadaFlowVo!=null){
                    rowMaxTotalMap.put(code,getStrDate(maxScadaFlowVo.getValue(),maxScadaFlowVo.getScadaTime(),simpleDateFormat));
                }
            });
        rowList.add(rowMinTotalMap);

        rowList.add(rowMaxTotalMap);

        //导出测试
        ExcelTool excelTool = new ExcelTool(exportTitle);
        List<ExcelColumn>  titleData=excelTool.columnTransformer(titleList,"id","pid","content","fielName","0");
        //excelTool.exportExcel(titleData,rowList,"D://Demo//test6.xlsx",true,true);
        excelTool.export(exportTitle, excelTool.exportWorkbook(titleData,rowList,true,true), HttpServletUtils.getServletResponse());

    }


    public  void exportScadaPipeIndexReport(List<ScadaIndexReportVo> list,String exportTitle) throws Exception {
        List<Map<String,String>> titleList=new ArrayList<>();

        //构建导出的表头，树形结构，包含标题
        Map<String,String> titleMap= new HashMap<>();
        //表头标题
        titleMap.put("id","-1");titleMap.put("pid","0");titleMap.put("content",exportTitle);titleMap.put("fielName",null);
        titleList.add(titleMap);
        Map<String,String> titleMap1= new HashMap<>();
        //多表头第一行固定，时间
        titleMap1.put("id","time");titleMap1.put("pid","-1");titleMap1.put("content","时间");titleMap1.put("fielName","time");
        titleList.add(titleMap1);

        //取list第一个数据构建动态表头
        List<WaterNameVo> waterNameVos=list.get(0).getWaterNameVos();
        waterNameVos.forEach(waterName->{

            Map<String,String> titleMapWater= new HashMap<>();
            //一级表头，水厂
            titleMapWater.put("id",waterName.getWaterId());
            titleMapWater.put("pid","-1");
            titleMapWater.put("content",waterName.getWaterName());
            titleMapWater.put("fielName",null);
            titleList.add(titleMapWater);
            //一级表头，水厂下二级表头，根据指标来构建
            waterName.getScadaList().forEach(scada->{
                Map<String,String> scadaMapWater= new HashMap<>();
                scadaMapWater.put("id",scada.getCode());scadaMapWater.put("pid","1");
                if(StringUtils.isNotBlank(scada.getUnit()) && !scada.getUnit().equals("null")){
                    scadaMapWater.put("content",scada.getVariableName()+"("+scada.getUnit()+")");
                }else{
                    scadaMapWater.put("content",scada.getVariableName());

                }
                scadaMapWater.put("fielName",scada.getCode());
                //fielName很重要，填充数据时候，根据fileName对应的值来的
                titleList.add(scadaMapWater);

            });




        });

        //填充行数据 rowList
        List<Map<String,String>> rowList=new ArrayList<>();
        //列表里面的数据放到listVo里面，方便找最大值，最小值所对应的时间
        List<ScadaFlowVo> listVo=new ArrayList<>();

        String simpleDateFormat="yyyy-MM-dd HH:mm:ss";
        //循环填充数据
        list.forEach(item->{
            Map<String,String> rowMap= new HashMap<>();
            rowMap.put("time",CalendarHelper.getFormatDate(item.getDateTime(),simpleDateFormat) );

            item.getWaterNameVos().forEach(waterNameVo->{
                waterNameVo.getScadaList().forEach(scadaFlowVo -> {
                    rowMap.put(scadaFlowVo.getCode(),getStrDate(scadaFlowVo.getValue(),scadaFlowVo.getScadaTime(),simpleDateFormat));
                    listVo.add(scadaFlowVo);

                });

            });
            rowList.add(rowMap);

        });


        Map<String,String> rowMinTotalMap=new HashMap<String,String>();
        Map<String,String> rowMaxTotalMap=new HashMap<String,String>();

        rowMinTotalMap.put("time","最小值");

        rowMaxTotalMap.put("time","最大值");

        Map<String,List<ScadaFlowVo>> listMaps=listVo.stream().collect(Collectors.groupingBy(ScadaFlowVo::getCode));
        //按照指标编码分组，取里面的最大值和最小值和对应的时间
        listMaps.forEach((code,voList)->{
            Comparator<ScadaFlowVo> scadaFlowVoComparator=Comparator.comparing(scada->Double.parseDouble(scada.getValue()));
            ScadaFlowVo maxScadaFlowVo=voList.stream().filter(item->StringUtil.isNotBlank(item.getValue())).max(scadaFlowVoComparator).orElse(null);
            ScadaFlowVo minScadaFlowVo=voList.stream().filter(item->StringUtil.isNotBlank(item.getValue())).min(scadaFlowVoComparator).orElse(null);
            if(minScadaFlowVo!=null){
                rowMinTotalMap.put(code,getStrDate(minScadaFlowVo.getValue(),minScadaFlowVo.getScadaTime(),simpleDateFormat));
            }
            if(maxScadaFlowVo!=null){
                rowMaxTotalMap.put(code,getStrDate(maxScadaFlowVo.getValue(),maxScadaFlowVo.getScadaTime(),simpleDateFormat));
            }
        });
        rowList.add(rowMinTotalMap);

        rowList.add(rowMaxTotalMap);

        //导出测试
        ExcelTool excelTool = new ExcelTool(exportTitle);
        List<ExcelColumn>  titleData=excelTool.columnTransformer(titleList,"id","pid","content","fielName","0");
        //  excelTool.exportExcel(titleData,rowList,"D://Demo//test6.xlsx",true,true);
        excelTool.export(exportTitle, excelTool.exportWorkbook(titleData,rowList,true,true), HttpServletUtils.getServletResponse());

    }


    private String getStrDate(String value,Date valueDate,String simpleDateFormat){
        String result="";
        if(StringUtils.isNotBlank(value) && valueDate!=null){
            result=value+"("+CalendarHelper.getFormatDate(valueDate,simpleDateFormat)+")";
        }
        return result;
    }
}
