package com.tofly.custom.controller;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.haiqiu.tools.common.utils.HttpServletUtils;
import com.tofly.common.ExcelTool.ExcelColumn;
import com.tofly.common.ExcelTool.ExcelTool;
import com.tofly.common.constant.CalendarHelper;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.custom.entity.ScadaFlowRecord;
import com.tofly.custom.entity.dto.ScadaFlowRecordDto;
import com.tofly.custom.entity.dto.ScadaWaterAmountDto;
import com.tofly.custom.entity.vo.ScadaWaterAmountVo;
import com.tofly.custom.service.ScadaFlowRecordService;
import com.tofly.custom.service.ScadaReportRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import java.util.*;


/**
 * @author JF
 * @date 2023-1-6
 * @description 自定义智慧综合展示相关
 */
@RestController
@RequestMapping("/scadaReportRecord")
@RefreshScope
@Api(tags = "Scada水厂数据报表")
public class ScadaReportRecordController {

 @Autowired
 private ScadaReportRecordService scadaReportRecordService;

    @Autowired
    private ScadaFlowRecordService scadaFlowRecordService;

    @PostMapping("/getScadaWaterAmount")
    @ApiOperation(value = "水厂水量时间范围内监测报表")
    public ResultRespone<List<ScadaWaterAmountVo>> getScadaWaterAmount(
            @Validated @RequestBody ScadaWaterAmountDto dto) {
        try {
            List<ScadaWaterAmountVo> list = scadaReportRecordService.getScadaWaterAmount(dto);
            if (dto.getExportStatus() != null && dto.getExportStatus() == 1) {
                exportScadaWaterAmount(list,dto.getExportTitle());
              return null;
            } else {
                return ResultRespone.success(list);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }


    @PostMapping("/getScadaFlowRecord")
    @ApiOperation(value = "水厂总瞬时流量报表")
    public ResultRespone<List<ScadaFlowRecord>> getScadaStaticFlowReport(
            @Validated  @RequestBody  ScadaFlowRecordDto dto
            ){
        String simpleDateFormat="yyyy-MM-dd";
        List<ScadaFlowRecord> list= scadaFlowRecordService.list(Wrappers.<ScadaFlowRecord>lambdaQuery()
                .eq(ScadaFlowRecord::getWaterType,dto.getWaterType())
                .ge(StringUtils.isNotBlank(dto.getStartDate()),ScadaFlowRecord::getRecordDay, CalendarHelper.ConvertStrToDate(dto.getStartDate(),simpleDateFormat) )
                .le(StringUtils.isNotBlank(dto.getEndDate()),ScadaFlowRecord::getRecordDay,CalendarHelper.ConvertStrToDate(dto.getEndDate(),simpleDateFormat) )

        );
        try {

            if (dto.getExportStatus() != null && dto.getExportStatus() == 1) {
                exportScadaStaticFlowReport(list,dto.getExportTitle());
                return null;
            } else {
                return ResultRespone.success(list);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }

    }

    private void exportScadaStaticFlowReport(List<ScadaFlowRecord> list,String exportTitle) {
//构建导出的表头，树形结构，包含标题

        List<Map<String,String>> titleList=new ArrayList<>();
        Map<String,String> titleMap= new HashMap<>();
        //表头标题
        if(StringUtils.isBlank(exportTitle)){
            exportTitle="自来水厂总瞬时流量监测报表";
        }


        titleMap.put("id","-1");titleMap.put("pid","0");titleMap.put("content",exportTitle);titleMap.put("fielName",null);
        titleList.add(titleMap);
        List<String> listTitle=new ArrayList<>();
        listTitle.add("recordDay@日期");
        listTitle.add("minFirstValue@最小值1");
        listTitle.add("minLastValue@最小值2");
        listTitle.add("maxFirstValue@最大值1");
        listTitle.add("maxLastValue@最大值2");

        listTitle.forEach(item->{
            String id=item.split("@")[0];
            String content=item.split("@")[1];
            String fielName=item.split("@")[0];
            Map<String,String> titleMap1= new HashMap<>();
            titleMap1.put("id",id);
            titleMap1.put("pid","-1");
            titleMap1.put("content",content);
            titleMap1.put("fielName",fielName);
            titleList.add(titleMap1);

        });

        List<Map<String,String>> rowList=new ArrayList<>();
        String simpleDateFormat="yyyy-MM-dd HH:mm";
        //循环填充数据
        list.forEach(item->{
            Map<String,String> rowMap= new HashMap<>();
            rowMap.put("recordDay",CalendarHelper.getFormatDate(item.getRecordDay(),"yyyy-MM-dd"));
            rowMap.put("minFirstValue",getStrDate(item.getMinFirstValue(),item.getMinFirstTime(),simpleDateFormat));
            rowMap.put("minLastValue",getStrDate(item.getMaxLastValue(),item.getMaxLastTime(),simpleDateFormat));
            rowMap.put("maxFirstValue",getStrDate(item.getMaxFirstValue(),item.getMaxFirstTime(),simpleDateFormat));
            rowMap.put("maxLastValue",getStrDate(item.getMaxLastValue(),item.getMaxLastTime(),simpleDateFormat));
            rowList.add(rowMap);

        });
        Map<String,String> rowMinTotalMap=new HashMap<String,String>();
        Map<String,String> rowMaxTotalMap=new HashMap<String,String>();

        rowMinTotalMap.put("recordDay","最小值");
        rowMaxTotalMap.put("recordDay","最大值");

        Comparator<ScadaFlowRecord> minFirstValueComparator=Comparator.comparing(scada->scada.getMinFirstValue());
        ScadaFlowRecord minFirstValueMin=list.stream().filter(item->item.getMinFirstValue()!=null).min(minFirstValueComparator).orElse(null);
        ScadaFlowRecord minFirstValueMax=list.stream().filter(item->item.getMinFirstValue()!=null).max(minFirstValueComparator).orElse(null);
        if(minFirstValueMin!=null){
            rowMinTotalMap.put("minFirstValue", getStrDate(minFirstValueMin.getMinFirstValue(),minFirstValueMin.getMinFirstTime(),simpleDateFormat));

        }
        if(minFirstValueMax!=null){
            rowMaxTotalMap.put("minFirstValue",getStrDate(minFirstValueMax.getMinFirstValue(),minFirstValueMax.getMinFirstTime(),simpleDateFormat));

        }

        Comparator<ScadaFlowRecord> minLaseValueComparator=Comparator.comparing(scada->scada.getMinLastValue());
        ScadaFlowRecord minLastValueMin=list.stream().filter(item->item.getMinLastValue()!=null).min(minLaseValueComparator).orElse(null);
        ScadaFlowRecord minLastValueMax=list.stream().filter(item->item.getMinLastValue()!=null).max(minLaseValueComparator).orElse(null);
        if(minLastValueMin!=null){
            rowMinTotalMap.put("minLastValue",getStrDate(minLastValueMin.getMinLastValue(),minLastValueMin.getMinLastTime(),simpleDateFormat));

        }
        if(minLastValueMax!=null){
            rowMaxTotalMap.put("minLastValue",getStrDate(minLastValueMax.getMinLastValue(),minLastValueMax.getMinLastTime(),simpleDateFormat));

        }


        Comparator<ScadaFlowRecord> maxFirstValueComparator=Comparator.comparing(scada->scada.getMaxFirstValue());
        ScadaFlowRecord maxFirstValueMin=list.stream().filter(item->item.getMaxFirstValue()!=null).min(maxFirstValueComparator).orElse(null);
        ScadaFlowRecord maxFirstValueMax=list.stream().filter(item->item.getMaxFirstValue()!=null).max(maxFirstValueComparator).orElse(null);
        if(maxFirstValueMin!=null){
            rowMinTotalMap.put("maxFirstValue",getStrDate(maxFirstValueMin.getMaxFirstValue(),maxFirstValueMin.getMaxFirstTime(),simpleDateFormat));

        }
        if(maxFirstValueMax!=null){
            rowMaxTotalMap.put("maxFirstValue",getStrDate(maxFirstValueMax.getMaxFirstValue(),maxFirstValueMax.getMaxFirstTime(),simpleDateFormat));

        }

        Comparator<ScadaFlowRecord> maxLastValueComparator=Comparator.comparing(scada->scada.getMaxLastValue());
        ScadaFlowRecord maxLastValueMin=list.stream().filter(item->item.getMaxLastValue()!=null).min(maxLastValueComparator).orElse(null);
        ScadaFlowRecord maxLastValueMax=list.stream().filter(item->item.getMaxLastValue()!=null).max(maxLastValueComparator).orElse(null);
        if(maxLastValueMin!=null){
            rowMinTotalMap.put("maxLastValue",getStrDate(maxLastValueMin.getMaxLastValue(),maxLastValueMin.getMaxLastTime(),simpleDateFormat));

        }
        if(maxLastValueMax!=null){
            rowMaxTotalMap.put("maxLastValue",getStrDate(maxLastValueMax.getMaxLastValue(),maxLastValueMax.getMaxLastTime(),simpleDateFormat));

        }
        rowList.add(rowMinTotalMap);
        rowList.add(rowMaxTotalMap);

        ExcelTool excelTool = new ExcelTool(exportTitle);
        List<ExcelColumn>  titleData= null;
        try {
            titleData = excelTool.columnTransformer(titleList,"id","pid","content","fielName","0");
              excelTool.export(exportTitle, excelTool.exportWorkbook(titleData,rowList,true,true), HttpServletUtils.getServletResponse());
            //  excelTool.exportExcel(titleData,rowList,"D://Demo//test6.xlsx",true,true);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }


    private String getStrDate(String value,Date valueDate,String simpleDateFormat){
       String result="";
        if(StringUtils.isNotBlank(value) && valueDate!=null){
            result=value+"("+CalendarHelper.getFormatDate(valueDate,simpleDateFormat)+")";
        }
        return result;
    }
    public  void  exportScadaWaterAmount(List<ScadaWaterAmountVo> list,String exportTitle){
        //构建导出的表头，树形结构，包含标题

        List<Map<String,String>> titleList=new ArrayList<>();
        Map<String,String> titleMap= new HashMap<>();
        //表头标题
        if(StringUtils.isBlank(exportTitle)){
            exportTitle="水厂水量监测报表导出";
        }

        titleMap.put("id","-1");titleMap.put("pid","0");titleMap.put("content",exportTitle);titleMap.put("fielName",null);
        titleList.add(titleMap);
        List<String> listTitle=new ArrayList<>();
        listTitle.add("dateStr@时间");
        listTitle.add("amountWater@总水量");
        listTitle.add("amountWater2@二水厂水量");
        listTitle.add("amountWater3@三水厂水量");

        listTitle.forEach(item->{
            String id=item.split("@")[0];
            String content=item.split("@")[1];
            String fielName=item.split("@")[0];
            Map<String,String> titleMap1= new HashMap<>();
            titleMap1.put("id",id);
            titleMap1.put("pid","-1");
            titleMap1.put("content",content);
            titleMap1.put("fielName",fielName);
            titleList.add(titleMap1);

        });
        String simpleDateFormat="yyyy-MM-dd HH:mm";
        List<Map<String,String>> rowList=new ArrayList<>();
        //循环填充数据
        list.forEach(item->{
            Map<String,String> rowMap= new HashMap<>();
            rowMap.put("dateStr",CalendarHelper.getFormatDate(item.getDateStr(),simpleDateFormat));
            rowMap.put("amountWater",item.getAmountWater());
            rowMap.put("amountWater2",item.getAmountWater2());
            rowMap.put("amountWater3",item.getAmountWater3());
            rowList.add(rowMap);

        });
        Map<String,String> rowMinTotalMap=new HashMap<String,String>();
        Map<String,String> rowMaxTotalMap=new HashMap<String,String>();

        rowMinTotalMap.put("dateStr","最小值");
        rowMaxTotalMap.put("dateStr","最大值");


        Comparator<ScadaWaterAmountVo> amountWaterComparator=Comparator.comparing(scada-> scada.getAmountWater());
        ScadaWaterAmountVo amountWaterMin=list.stream().filter(item->item.getAmountWater()!=null).min(amountWaterComparator).orElse(null);
        ScadaWaterAmountVo amountWaterMax=list.stream().filter(item->item.getAmountWater()!=null).max(amountWaterComparator).orElse(null);
        if(amountWaterMin!=null){
            rowMinTotalMap.put("amountWater", getStrDate(amountWaterMin.getAmountWater(),amountWaterMin.getDateStr(),simpleDateFormat));

        }
        if(amountWaterMax!=null){
            rowMaxTotalMap.put("amountWater",getStrDate(amountWaterMax.getAmountWater(),amountWaterMax.getDateStr(),simpleDateFormat));

        }

        Comparator<ScadaWaterAmountVo> amountWater2Comparator=Comparator.comparing(scada->scada.getAmountWater2());
        ScadaWaterAmountVo amountWater2Min=list.stream().filter(item->item.getAmountWater2()!=null).min(amountWater2Comparator).orElse(null);
        ScadaWaterAmountVo amountWater2Max=list.stream().filter(item->item.getAmountWater2()!=null).max(amountWater2Comparator).orElse(null);
        if(amountWater2Min!=null){
            rowMinTotalMap.put("amountWater2", getStrDate(amountWater2Min.getAmountWater2(),amountWater2Min.getDateStr(),simpleDateFormat));

        }
        if(amountWater2Max!=null){
            rowMaxTotalMap.put("amountWater2",getStrDate(amountWater2Max.getAmountWater2(),amountWater2Max.getDateStr(),simpleDateFormat));

        }
        Comparator<ScadaWaterAmountVo> amountWater3Comparator=Comparator.comparing(scada->scada.getAmountWater3());
        ScadaWaterAmountVo amountWater3Min=list.stream().filter(item->item.getAmountWater3()!=null).min(amountWater3Comparator).orElse(null);
        ScadaWaterAmountVo amountWater3Max=list.stream().filter(item->item.getAmountWater3()!=null).max(amountWater3Comparator).orElse(null);
        if(amountWater3Min!=null){
            rowMinTotalMap.put("amountWater3", getStrDate(amountWater3Min.getAmountWater3(),amountWater3Min.getDateStr(),simpleDateFormat));

        }
        if(amountWater3Max!=null){
            rowMaxTotalMap.put("amountWater3",getStrDate(amountWater3Max.getAmountWater3(),amountWater3Max.getDateStr(),simpleDateFormat));

        }

        ExcelTool excelTool = new ExcelTool(exportTitle);
        List<ExcelColumn>  titleData= null;
        try {
            titleData = excelTool.columnTransformer(titleList,"id","pid","content","fielName","0");
            excelTool.export(exportTitle, excelTool.exportWorkbook(titleData,rowList,true,true), HttpServletUtils.getServletResponse());
            // excelTool.exportExcel(titleData,rowList,"D://Demo//test11.xlsx",true,true);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }
}
