package com.tofly.custom.service.impl;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.constant.CalendarHelper;
import com.tofly.custom.entity.ScadaFlowRecord;
import com.tofly.custom.entity.ScadaPipeReportRecord;
import com.tofly.custom.entity.ScadaReportRecord;
import com.tofly.custom.entity.dto.ScadaWaterAmountDto;
import com.tofly.custom.entity.vo.ScadaFlowRecordJobVo;
import com.tofly.custom.entity.vo.ScadaFlowVo;
import com.tofly.custom.entity.vo.ScadaReportRecordVo;
import com.tofly.custom.entity.vo.ScadaWaterAmountVo;
import com.tofly.custom.mapper.CustomDisplayMapper;
import com.tofly.custom.mapper.ScadaFlowRecordMapper;
import com.tofly.custom.mapper.ScadaReportRecordMapper;
import com.tofly.custom.service.ScadaPipeReportRecordService;
import com.tofly.custom.service.ScadaReportRecordService;
import com.tofly.scada.ScadaApplication;
import com.tofly.scada.common.DateTimeUtil;
import com.tofly.scada.entity.Allocation;
import com.tofly.scada.entity.Scada;
import com.tofly.scada.mapper.AllocationMapper;
import com.tofly.scada.mapper.ScadaMapper;
import com.tofly.scada.service.WaterReportService;
import com.tofly.water.entity.WaterReport;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author haiqiu
 * @description 针对表【TF_INDEX_ALLOCATION(指标配置)】的数据库操作Service实现
 * @createDate 2022-05-12 09:48:47
 */
@Service("scadaReportRecord")
public class ScadaReportRecordServiceImpl extends ServiceImpl<ScadaReportRecordMapper, ScadaReportRecord> implements ScadaReportRecordService
{
    @Autowired
    @Lazy
    private  CustomDisplayMapper customDisplayMapper;
    @Autowired
    @Lazy
    private  AllocationMapper allocationMapper;

    @Autowired
    @Lazy
    private  ScadaReportRecordService scadaReportRecordService;

    @Autowired
    @Lazy
    private ScadaPipeReportRecordService scadaPipeReportRecordService;

    @Autowired
    @Lazy
    private ScadaMapper scadaMapper;

    @Autowired
    @Lazy
    private ScadaFlowRecordMapper scadaFlowRecordMapper;

    @Autowired
    @Lazy
    private WaterReportService waterReportService;

    /**
     * 水厂,污水厂总瞬时流量(出)
     */
    @Value("${waterSupplyChu.codesFlow}")
    private String waterSupplyChuFlow;

    /**
     * 污水厂总瞬时流量(出)
     */
    @Value("${waterSupplyChu.codesWsFlow}")
    private String waterSupplyChuWsFlow;

    /**
     * 水厂,污水厂总瞬时流量(进)
     */
    @Value("${waterSupplyJin.codesFlow}")
    private String waterSupplyJinFlow;

    /**
     * 污水厂总瞬时流量
     */
    @Value("${waterSupplyJin.codesWsFlow}")
    private String waterSupplyJinWsFlow;


    /**
     * 水厂/污水厂数据加入缓存表XXlJob
     */
    @Async
    @Override
    public void getWaterFlowCommXXLJobCode(String indexType,String codes, Integer minutes,String startTime,String endTime) {
        //结果返回放到list中
        //特定的指标计算水厂的总瞬时量
        List<String >listType=null;
        if(StringUtils.isNotBlank(indexType)){
            listType=Arrays.asList(indexType.split(","));
        }

        List<String >listCodes=null;
        if(StringUtils.isNotBlank(codes)){
            listCodes=Arrays.asList(codes.split(","));
        }
        //区分水厂和污水厂
        List<Allocation>  list_result=allocationMapper.getIndexListByType(null,listType,listCodes);
        List<Allocation> list=list_result.stream().filter(item->!item.getType().equals("18")).collect(Collectors.toList());
        String simpleDateFormatMinutes = "yyyy-MM-dd HH:mm";
        List<String> listDates= CalendarHelper.getMinutesBetweenDate(startTime,endTime, minutes, simpleDateFormatMinutes);

        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                6,
                12,
                480,
                TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(1000),
                (r) -> {
                    Thread t = new Thread(r);
                    t.setName("thread-" + ((int) (Math.random() * 1000)));
                    return t;
                },
                new ThreadPoolExecutor.CallerRunsPolicy());


        listDates.forEach(date->{
            threadPoolExecutor.execute(()->getWaterFlowComm(list,minutes,date));
        });

       /* list.forEach(allocation->{
            threadPoolExecutor.execute(()->getWaterFlowComm(allocation,minutes,listDates));

        });*/



    }

    /**
     * 管网数据加入缓存表XXlJob
     * @param indexType
     * @param codes
     * @param minutes
     * @param startTime
     * @param endTime
     */
    @Async
    @Override
    public void getPipeFlowCommXXLJobCode(String indexType,String codes, Integer minutes,String startTime,String endTime) {
        //结果返回放到list中
        //特定的指标计算水厂的总瞬时量
        //结果返回放到list中
        //特定的指标计算水厂的总瞬时量
        List<String >listType=null;
        if(StringUtils.isBlank(indexType)){
            indexType="0,11";
        }

        listType=Arrays.asList(indexType.split(","));

        List<String >listCodes=null;
        if(StringUtils.isNotBlank(codes)){
            listCodes=Arrays.asList(codes.split(","));
        }
        //区分水厂和污水厂
        List<Allocation>  list_result=allocationMapper. getPipeIndexListByType(listType,listCodes);
        List<Allocation> list=list_result.stream().filter(item->!item.getType().equals("18")).collect(Collectors.toList());
        String simpleDateFormatMinutes = "yyyy-MM-dd HH:mm";
        List<String> listDates= CalendarHelper.getMinutesBetweenDate(startTime,endTime, minutes, simpleDateFormatMinutes);

        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                6,
                12,
                480,
                TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(1000),
                (r) -> {
                    Thread t = new Thread(r);
                    t.setName("thread-" + ((int) (Math.random() * 1000)));
                    return t;
                },
                new ThreadPoolExecutor.CallerRunsPolicy());

        listDates.forEach(date->{
            threadPoolExecutor.execute(()->getPipeFlowComm(list,minutes,date));

        });
       /* list.forEach(allocation->{
            threadPoolExecutor.execute(()->getPipeFlowComm(allocation,minutes,listDates));

        });*/


    }

    /**
     * 水厂/污水厂数据加入缓存表XXlJob
     */
    @Async
    public void getWaterFlowComm(List<Allocation> allocationList, Integer minutes, String date) {
        List<String> codesChu= Arrays.asList(waterSupplyChuFlow.split(","));
        List<String> codesWsChu= Arrays.asList(waterSupplyChuWsFlow.split(","));
        List<String> codesWsJin= Arrays.asList(waterSupplyJinWsFlow.split(","));
        List<String> codesJin= Arrays.asList(waterSupplyJinFlow.split(","));
        String simpleDateFormatMinutes = "yyyy-MM-dd HH:mm";
        allocationList.forEach(allocation->{
            try {
                LocalDateTime localTime = DateTimeUtil.fromLong2LocalDateTime(CalendarHelper.ConvertStrToDate(date, simpleDateFormatMinutes).getTime());
                Date LocalBeginTime = DateTimeUtil.convertLDTToDate(localTime);
                Date localEndTime = DateTimeUtil.convertLDTToDate(localTime.plusMinutes(minutes).plusSeconds(-1));
                ScadaReportRecord scadaReportRecord = this.baseMapper.getWaterFlowCommXXLJob(allocation.getType(), allocation.getVariableCode(), LocalBeginTime, localEndTime);
                String totalFlow = "";
                String totalFlowJin = "";
                if (allocation.getWaterType().equals("2")) {
                    totalFlow = customDisplayMapper.getTotalFlow(codesWsChu, LocalBeginTime, localEndTime);
                    totalFlowJin = customDisplayMapper.getTotalFlow(codesWsJin, LocalBeginTime, localEndTime);


                } else {
                    totalFlow = customDisplayMapper.getTotalFlow(codesChu, LocalBeginTime, localEndTime);
                    totalFlowJin = customDisplayMapper.getTotalFlow(codesJin, LocalBeginTime, localEndTime);
                }

                if (StringUtils.isNotBlank(totalFlow) && totalFlow.startsWith(".")) {
                    totalFlow = "0" + totalFlow;
                }
                if (StringUtils.isNotBlank(totalFlowJin) && totalFlowJin.startsWith(".")) {
                    totalFlowJin = "0" + totalFlowJin;
                }
                //累计量单独计算
                if (allocation.getType().equals("19") && scadaReportRecord != null) {
                    Scada scadaCheck = scadaMapper.selectLastOne(allocation.getVariableCode(), LocalBeginTime);
                    String minValue = scadaReportRecord.getMinValue();
                    if (scadaCheck != null) {
                        minValue = scadaCheck.getValue();
                    }
                    BigDecimal fold = new BigDecimal(scadaReportRecord.getMaxValue()).subtract(new BigDecimal(minValue));
                    scadaReportRecord.setFold(getRoundStr(String.valueOf(fold), allocation.getAccuracy()));

                }
                if (scadaReportRecord == null) {
                    scadaReportRecord = new ScadaReportRecord();

                }
                scadaReportRecord.setType(minutes);
                scadaReportRecord.setCode(allocation.getVariableCode());
                scadaReportRecord.setStartTime(LocalBeginTime);
                scadaReportRecord.setEndTime(localEndTime);
                scadaReportRecord.setTotalFlow(totalFlow);
                scadaReportRecord.setTotalFlowJin(totalFlowJin);
                scadaReportRecord.setUpdateTime(new Date());
                scadaReportRecord.setFirstValue(getRoundStr(scadaReportRecord.getFirstValue(), allocation.getAccuracy()));
                scadaReportRecord.setLastValue(getRoundStr(scadaReportRecord.getLastValue(), allocation.getAccuracy()));
                scadaReportRecord.setMaxValue(getRoundStr(scadaReportRecord.getMaxValue(), allocation.getAccuracy()));
                scadaReportRecord.setMinValue(getRoundStr(scadaReportRecord.getMinValue(), allocation.getAccuracy()));
                scadaReportRecord.setAvgValue(getRoundStr(scadaReportRecord.getAvgValue(), 2));
                ScadaReportRecord scadaReportOld = scadaReportRecordService.list(Wrappers.<ScadaReportRecord>lambdaQuery()
                        .eq(ScadaReportRecord::getType, minutes)
                        .eq(ScadaReportRecord::getCode, allocation.getVariableCode())
                        .eq(ScadaReportRecord::getStartTime, LocalBeginTime)).stream().findFirst().orElse(null);
                if (scadaReportOld != null) {
                    scadaReportRecord.setId(scadaReportOld.getId());
                }
                scadaReportRecordService.saveOrUpdate(scadaReportRecord);
            }catch (Exception exception){

            }
        });

    }

    /**
     * 水厂/污水厂数据加入缓存表XXlJob
     */
    @Async
    public void getWaterFlowComm(Allocation allocation, Integer minutes, List<String> listDates) {
        List<String> codesChu= Arrays.asList(waterSupplyChuFlow.split(","));
        List<String> codesWsChu= Arrays.asList(waterSupplyChuWsFlow.split(","));
        List<String> codesWsJin= Arrays.asList(waterSupplyJinWsFlow.split(","));
        List<String> codesJin= Arrays.asList(waterSupplyJinFlow.split(","));
        String simpleDateFormatMinutes = "yyyy-MM-dd HH:mm";
        listDates.forEach(date->{

            try {
                LocalDateTime localTime = DateTimeUtil.fromLong2LocalDateTime(CalendarHelper.ConvertStrToDate(date, simpleDateFormatMinutes).getTime());
                Date LocalBeginTime = DateTimeUtil.convertLDTToDate(localTime);
                Date localEndTime = DateTimeUtil.convertLDTToDate(localTime.plusMinutes(minutes).plusSeconds(-1));
                ScadaReportRecord scadaReportRecord = this.baseMapper.getWaterFlowCommXXLJob(allocation.getType(), allocation.getVariableCode(), LocalBeginTime, localEndTime);

                String totalFlow = "";
                String totalFlowJin = "";
                if (allocation.getWaterType().equals("2")) {
                    totalFlow = customDisplayMapper.getTotalFlow(codesWsChu, LocalBeginTime, localEndTime);
                    totalFlowJin = customDisplayMapper.getTotalFlow(codesWsJin, LocalBeginTime, localEndTime);


                } else {
                    totalFlow = customDisplayMapper.getTotalFlow(codesChu, LocalBeginTime, localEndTime);
                    totalFlowJin = customDisplayMapper.getTotalFlow(codesJin, LocalBeginTime, localEndTime);
                }

                if (StringUtils.isNotBlank(totalFlow) && totalFlow.startsWith(".")) {
                    totalFlow = "0" + totalFlow;
                }
                if (StringUtils.isNotBlank(totalFlowJin) && totalFlowJin.startsWith(".")) {
                    totalFlowJin = "0" + totalFlowJin;
                }
                //累计量单独计算
                if (allocation.getType().equals("19") && scadaReportRecord != null) {
                    Scada scadaCheck = scadaMapper.selectLastOne(allocation.getVariableCode(), LocalBeginTime);
                    String minValue = scadaReportRecord.getMinValue();
                    if (scadaCheck != null) {
                        minValue = scadaCheck.getValue();
                    }
                    BigDecimal fold = new BigDecimal(scadaReportRecord.getMaxValue()).subtract(new BigDecimal(minValue));
                    scadaReportRecord.setFold(getRoundStr(String.valueOf(fold), allocation.getAccuracy()));

                }
                if (scadaReportRecord == null) {
                    scadaReportRecord = new ScadaReportRecord();

                }
                scadaReportRecord.setType(minutes);
                scadaReportRecord.setCode(allocation.getVariableCode());
                scadaReportRecord.setStartTime(LocalBeginTime);
                scadaReportRecord.setEndTime(localEndTime);
                scadaReportRecord.setTotalFlow(totalFlow);
                scadaReportRecord.setTotalFlowJin(totalFlowJin);
                scadaReportRecord.setUpdateTime(new Date());
                scadaReportRecord.setFirstValue(getRoundStr(scadaReportRecord.getFirstValue(), allocation.getAccuracy()));
                scadaReportRecord.setLastValue(getRoundStr(scadaReportRecord.getLastValue(), allocation.getAccuracy()));
                scadaReportRecord.setMaxValue(getRoundStr(scadaReportRecord.getMaxValue(), allocation.getAccuracy()));
                scadaReportRecord.setMinValue(getRoundStr(scadaReportRecord.getMinValue(), allocation.getAccuracy()));
                scadaReportRecord.setAvgValue(getRoundStr(scadaReportRecord.getAvgValue(), 2));
                ScadaReportRecord scadaReportOld = scadaReportRecordService.list(Wrappers.<ScadaReportRecord>lambdaQuery()
                        .eq(ScadaReportRecord::getType, minutes)
                        .eq(ScadaReportRecord::getCode, allocation.getVariableCode())
                        .eq(ScadaReportRecord::getStartTime, LocalBeginTime)).stream().findFirst().orElse(null);
                if (scadaReportOld != null) {
                    scadaReportRecord.setId(scadaReportOld.getId());
                }
                scadaReportRecordService.saveOrUpdate(scadaReportRecord);
            }catch (Exception exception){

            }
            });

    }

    /**
     * 同步管网数据
     */
    @Async
    public void getPipeFlowComm(List<Allocation> allocationList, Integer minutes,String date) {
        String simpleDateFormatMinutes = "yyyy-MM-dd HH:mm";
        allocationList.forEach(allocation->{
            try {
                LocalDateTime localTime = DateTimeUtil.fromLong2LocalDateTime(CalendarHelper.ConvertStrToDate(date, simpleDateFormatMinutes).getTime());
                Date LocalBeginTime = DateTimeUtil.convertLDTToDate(localTime);
                Date localEndTime = DateTimeUtil.convertLDTToDate(localTime.plusMinutes(minutes).plusSeconds(-1));
                ScadaReportRecord scadaReportRecordComm = this.baseMapper.getWaterFlowCommXXLJob(allocation.getType(), allocation.getVariableCode(), LocalBeginTime, localEndTime);

                ScadaPipeReportRecord scadaPipeReportRecord = new ScadaPipeReportRecord();
                if(scadaReportRecordComm!=null){
                    BeanUtils.copyProperties(scadaReportRecordComm, scadaPipeReportRecord);
                }

                //累计量单独计算
                if (allocation.getType().equals("19") && scadaReportRecordComm != null) {
                    Scada scadaCheck = scadaMapper.selectLastOne(allocation.getVariableCode(), LocalBeginTime);
                    String minValue = scadaPipeReportRecord.getMinValue();
                    if (scadaCheck != null) {
                        minValue = scadaCheck.getValue();
                    }
                    BigDecimal fold = new BigDecimal(scadaPipeReportRecord.getMaxValue()).subtract(new BigDecimal(minValue));
                    scadaPipeReportRecord.setFold(getRoundStr(String.valueOf(fold), allocation.getAccuracy()));

                }

                scadaPipeReportRecord.setType(minutes);
                scadaPipeReportRecord.setCode(allocation.getVariableCode());
                scadaPipeReportRecord.setStartTime(LocalBeginTime);
                scadaPipeReportRecord.setEndTime(localEndTime);
                scadaPipeReportRecord.setUpdateTime(new Date());
                scadaPipeReportRecord.setFirstValue(getRoundStr(scadaPipeReportRecord.getFirstValue(), allocation.getAccuracy()));
                scadaPipeReportRecord.setLastValue(getRoundStr(scadaPipeReportRecord.getLastValue(), allocation.getAccuracy()));
                scadaPipeReportRecord.setMaxValue(getRoundStr(scadaPipeReportRecord.getMaxValue(), allocation.getAccuracy()));
                scadaPipeReportRecord.setMinValue(getRoundStr(scadaPipeReportRecord.getMinValue(), allocation.getAccuracy()));

                ScadaPipeReportRecord scadaPipeReportRecordOld = scadaPipeReportRecordService.list(Wrappers.<ScadaPipeReportRecord>lambdaQuery()
                        .eq(ScadaPipeReportRecord::getType, minutes)
                        .eq(ScadaPipeReportRecord::getCode, allocation.getVariableCode())
                        .eq(ScadaPipeReportRecord::getStartTime, LocalBeginTime)).stream().findFirst().orElse(null);
                if (scadaPipeReportRecordOld != null) {
                    scadaPipeReportRecord.setId(scadaPipeReportRecordOld.getId());
                }
                scadaPipeReportRecordService.saveOrUpdate(scadaPipeReportRecord);
            }catch (Exception exception){

            }
        });

    }


    /**
     * 同步管网数据
     */
    @Async
    public void getPipeFlowComm(Allocation allocation, Integer minutes, List<String> listDates) {

        String simpleDateFormatMinutes = "yyyy-MM-dd HH:mm";
        listDates.forEach(date->{

            try {
                LocalDateTime localTime = DateTimeUtil.fromLong2LocalDateTime(CalendarHelper.ConvertStrToDate(date, simpleDateFormatMinutes).getTime());
                Date LocalBeginTime = DateTimeUtil.convertLDTToDate(localTime);
                Date localEndTime = DateTimeUtil.convertLDTToDate(localTime.plusMinutes(minutes).plusSeconds(-1));
                ScadaReportRecord scadaReportRecordComm = this.baseMapper.getWaterFlowCommXXLJob(allocation.getType(), allocation.getVariableCode(), LocalBeginTime, localEndTime);

                ScadaPipeReportRecord scadaPipeReportRecord = new ScadaPipeReportRecord();
               if(scadaReportRecordComm!=null){
                   BeanUtils.copyProperties(scadaReportRecordComm, scadaPipeReportRecord);
               }

                //累计量单独计算
                if (allocation.getType().equals("19") && scadaReportRecordComm != null) {
                    Scada scadaCheck = scadaMapper.selectLastOne(allocation.getVariableCode(), LocalBeginTime);
                    String minValue = scadaPipeReportRecord.getMinValue();
                    if (scadaCheck != null) {
                        minValue = scadaCheck.getValue();
                    }
                    BigDecimal fold = new BigDecimal(scadaPipeReportRecord.getMaxValue()).subtract(new BigDecimal(minValue));
                    scadaPipeReportRecord.setFold(getRoundStr(String.valueOf(fold), allocation.getAccuracy()));

                }

                scadaPipeReportRecord.setType(minutes);
                scadaPipeReportRecord.setCode(allocation.getVariableCode());
                scadaPipeReportRecord.setStartTime(LocalBeginTime);
                scadaPipeReportRecord.setEndTime(localEndTime);
                scadaPipeReportRecord.setUpdateTime(new Date());
                scadaPipeReportRecord.setFirstValue(getRoundStr(scadaPipeReportRecord.getFirstValue(), allocation.getAccuracy()));
                scadaPipeReportRecord.setLastValue(getRoundStr(scadaPipeReportRecord.getLastValue(), allocation.getAccuracy()));
                scadaPipeReportRecord.setMaxValue(getRoundStr(scadaPipeReportRecord.getMaxValue(), allocation.getAccuracy()));
                scadaPipeReportRecord.setMinValue(getRoundStr(scadaPipeReportRecord.getMinValue(), allocation.getAccuracy()));

                ScadaPipeReportRecord scadaPipeReportRecordOld = scadaPipeReportRecordService.list(Wrappers.<ScadaPipeReportRecord>lambdaQuery()
                        .eq(ScadaPipeReportRecord::getType, minutes)
                        .eq(ScadaPipeReportRecord::getCode, allocation.getVariableCode())
                        .eq(ScadaPipeReportRecord::getStartTime, LocalBeginTime)).stream().findFirst().orElse(null);
                if (scadaPipeReportRecordOld != null) {
                    scadaPipeReportRecord.setId(scadaPipeReportRecordOld.getId());
                }
                scadaPipeReportRecordService.saveOrUpdate(scadaPipeReportRecord);
            }catch (Exception exception){

            }
        });

    }

    @Async
    @Override
    /**
     * 水厂总瞬时流量加入缓存数据
     */
    public void getScadaFlowCommXXLJobCode(String startDay,String endDay) {
        Integer minutes=5;
        String waterType="1";
     List<String> list= CalendarHelper.getDayBetweenDate(startDay,endDay,"yyyy-MM-dd");
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                4,
                8,
                480,
                TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(1000),
                (r) -> {
                    Thread t = new Thread(r);
                    t.setName("thread-" + ((int) (Math.random() * 1000)));
                    return t;
                },
                new ThreadPoolExecutor.CallerRunsPolicy());

        list.forEach(dateDay->{
            threadPoolExecutor.execute(()->getScadaFlowRecordComm(dateDay,minutes,waterType));

        });

    }


    /**
     * 水厂水量时间范围内监测报表
     */
    @Override
    public  List<ScadaWaterAmountVo>  getScadaWaterAmount(ScadaWaterAmountDto scadaWaterAmountDto){
        String startTime=StringUtils.isBlank(scadaWaterAmountDto.getStartTime())?"00:00:00":scadaWaterAmountDto.getStartTime();
        String endTime=StringUtils.isBlank(scadaWaterAmountDto.getEndTime())?"23:59:59":scadaWaterAmountDto.getEndTime();
        return baseMapper.getScadaWaterAmount(scadaWaterAmountDto.getStartDate(),scadaWaterAmountDto.getEndDate(),startTime,endTime);

    }

    /**
     * 水厂总瞬时流量加入缓存数据
     */
    @Async
    public  void  getScadaFlowRecordComm(String dateDay, Integer minutes,String waterType){
        String type="11";
        String codeSc2="SC2_Chu_Total_FT";
        String codeSc3="SC3_Chu_FT";
        List<String> codesList=new ArrayList<>();
        codesList.add(codeSc2);
        codesList.add(codeSc3);
        String dayStart="00:00";
        String dayEnd="23:59";
        String simpleDateFormatMinutes = "yyyy-MM-dd HH:mm";

        int roundNum=2;
        List<ScadaFlowRecordJobVo> list=new ArrayList<>();

        List<String> listDates= CalendarHelper.getMinutesBetweenDate(dateDay+" "+dayStart,dateDay+" "+dayEnd, minutes, simpleDateFormatMinutes);

        listDates.forEach(date->{
            LocalDateTime localTime = DateTimeUtil.fromLong2LocalDateTime(CalendarHelper.ConvertStrToDate(date, simpleDateFormatMinutes).getTime());
            Date LocalBeginTime = DateTimeUtil.convertLDTToDate(localTime);
            Date localEndTime = DateTimeUtil.convertLDTToDate(localTime.plusMinutes(minutes).plusSeconds(-1));
            ScadaReportRecord scadaReportRecordSc2 = this.baseMapper.getWaterFlowCommXXLJob(type, codeSc2, LocalBeginTime, localEndTime);
            ScadaReportRecord scadaReportRecordSc3 = this.baseMapper.getWaterFlowCommXXLJob(type, codeSc3, LocalBeginTime, localEndTime);

            double Sc2Value=0L;
            double Sc3Value=0L;
            if(scadaReportRecordSc2!=null && StringUtils.isNotBlank(scadaReportRecordSc2.getFirstValue())){
                Sc2Value=Double.parseDouble(scadaReportRecordSc2.getFirstValue());

            }
            if(scadaReportRecordSc3!=null && StringUtils.isNotBlank(scadaReportRecordSc3.getFirstValue())){
                Sc3Value=Double.parseDouble(scadaReportRecordSc3.getFirstValue());

            }
            ScadaFlowRecordJobVo scadaFlowRecordJobVo=new ScadaFlowRecordJobVo();
            scadaFlowRecordJobVo.setDate(CalendarHelper.ConvertStrToDate(date,simpleDateFormatMinutes));
            scadaFlowRecordJobVo.setValue(Sc2Value+Sc3Value);
            list.add(scadaFlowRecordJobVo);
        });

        List<ScadaFlowRecordJobVo> listMax= list.stream().sorted(Comparator.comparing(ScadaFlowRecordJobVo::getValue,Comparator.reverseOrder()).thenComparing(ScadaFlowRecordJobVo::getDate)).collect(Collectors.toList());
        List<ScadaFlowRecordJobVo> listMin= list.stream().sorted(Comparator.comparing(ScadaFlowRecordJobVo::getValue).thenComparing(ScadaFlowRecordJobVo::getDate)).collect(Collectors.toList());

        ScadaFlowRecord scadaFlowRecord= scadaFlowRecordMapper.selectList(Wrappers.<ScadaFlowRecord>lambdaQuery()
                        .eq(ScadaFlowRecord::getWaterType,waterType)
                        .eq(ScadaFlowRecord::getRecordDay,CalendarHelper.ConvertStrToDate(dateDay,"yyyy-MM-dd")))
                .stream().findFirst().orElse(null);
        boolean isAdd=false;
        if(scadaFlowRecord==null){
            scadaFlowRecord=new ScadaFlowRecord();
            isAdd=true;
        }else{
            scadaFlowRecord.setId(scadaFlowRecord.getId());
        }
        scadaFlowRecord.setRecordDay(CalendarHelper.ConvertStrToDate(dateDay,"yyyy-MM-dd"));
        scadaFlowRecord.setMaxFirstValue(getRoundStr(listMax.get(0).getValue()+"",roundNum));
        scadaFlowRecord.setMaxFirstTime(listMax.get(0).getDate());
        scadaFlowRecord.setMaxLastValue(getRoundStr(listMax.get(1).getValue()+"",roundNum));
        scadaFlowRecord.setMaxLastTime(listMax.get(1).getDate());

        scadaFlowRecord.setMinFirstValue(getRoundStr(listMin.get(0).getValue()+"",roundNum));
        scadaFlowRecord.setMinFirstTime(listMin.get(0).getDate());
        scadaFlowRecord.setMinLastValue(getRoundStr(listMin.get(1).getValue()+"",roundNum));
        scadaFlowRecord.setMinLastTime(listMin.get(1).getDate());
        scadaFlowRecord.setWaterType(waterType);
        if(isAdd){
            scadaFlowRecordMapper.insert(scadaFlowRecord);
        }else {
            scadaFlowRecordMapper.updateById(scadaFlowRecord);
        }

    }



    /**
     * 污水厂减排报表数据加入到减排表
     * @param startDay
     * @param endDay
     */
    @Async
    @Override
    public void getScadaWaterReportXXlJobCode(String startDay,String endDay) {
        List<String> list= CalendarHelper.getDayBetweenDate(startDay,endDay,"yyyy-MM-dd");
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                4,
                8,
                480,
                TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(1000),
                (r) -> {
                    Thread t = new Thread(r);
                    t.setName("thread-" + ((int) (Math.random() * 1000)));
                    return t;
                },
                new ThreadPoolExecutor.CallerRunsPolicy());
         list.forEach(dateDay->{
            threadPoolExecutor.execute(()->getScadaWaterReportComm(dateDay));

        });

    }

    /**
     * 污水厂减排报表数据加入到减排表
     * @param
     * @param
     */
    @Async
    public  void getScadaWaterReportComm(String date){
        Date recordDay=CalendarHelper.ConvertStrToDate(date,"yyyy-MM-dd");
        //总流量
        String listTotalJinCodes="WSC1_Jin_TFT1,WSC1_Jin_TFT2,WSC2_Jin_TFT";
        String listTotalChuCodes="WSC1_Chu_TFT,WSC2_Chu_TFT";
        //cod
        String codJinCodes="WSC1_Jin_COD,WSC2_Jin_COD";
        String codChuCodes="WSC1_Chu_COD,WSC2_Chu_COD";
        //氨氮
        String adJinCodes="WSC1_Jin_NH4,WSC2_Jin_NHN";
        String adChuCodes="WSC1_Chu_NH4,WSC2_Chu_NHN";
        //ss
        String ssJinCodes="WSC1_Jin_SS,WSC2_Jin_SS";
        String ssChuCodes="WSC1_Chu_SS,WSC2_Chu_SS";
        //总氮
        String tnJinCodes="WSC1_Jin_TN,WSC2_Jin_TN";
        String  tnChuCodes="WSC1_Chu_TN,WSC2_Chu_TN";

        //总磷
        String tpJinCodes="WSC1_Jin_TP,WSC2_Jin_TP";
        String tpChuCodes="WSC1_Chu_TP,WSC2_Chu_TP";
        String codes=listTotalJinCodes.concat(",")
                .concat(listTotalChuCodes).concat(",")
                .concat(codJinCodes).concat(",")
                .concat(codChuCodes).concat(",")
                .concat(adJinCodes).concat(",")
                .concat(adChuCodes).concat(",")
                .concat(ssJinCodes).concat(",")
                .concat(ssChuCodes).concat(",")
                .concat(tnJinCodes).concat(",")
                .concat(tnChuCodes).concat(",")
                .concat(tpJinCodes).concat(",")
                .concat(tpJinCodes).concat(",")
                .concat(tpChuCodes);

        List<String> listCodes=Arrays.asList(codes.split(","));
       List<WaterReport> waterReports=new ArrayList<>();
        List<ScadaReportRecordVo>  voList=baseMapper.getScadaReportRecord(recordDay ,listCodes);
        Map<Long,List<ScadaReportRecordVo>> listMap=voList.stream().collect(Collectors.groupingBy(ScadaReportRecordVo::getWaterId));
      List<WaterReport> waterReportList= waterReportService.list(Wrappers.<WaterReport>lambdaQuery()
                .eq(WaterReport::getRecordDate,recordDay));

      Date nowDate=new Date();
        listMap.forEach((waterId,recordVoList)->{
            String totalFlowNow=getCodeValue(recordVoList,listTotalJinCodes,false);
            String dealWaterVolume=getCodeValue(recordVoList,listTotalChuCodes,false);
            String codWaterIn=getCodeValue(recordVoList,codJinCodes,true);
            String codWaterOut=getCodeValue(recordVoList,codChuCodes,true);
            String adWaterIn=getCodeValue(recordVoList,adJinCodes,true);
            String adWaterOut=getCodeValue(recordVoList,adChuCodes,true);
            String ssWaterIn=getCodeValue(recordVoList,ssJinCodes,true);
            String ssWaterOut=getCodeValue(recordVoList,ssChuCodes,true);
            String zdWaterIn=getCodeValue(recordVoList,tnJinCodes,true);
            String zdWaterOut=getCodeValue(recordVoList,tnChuCodes,true);
            String zlWaterIn=getCodeValue(recordVoList,tpJinCodes,true);
            String zlWaterOut=getCodeValue(recordVoList,tpChuCodes,true);
            WaterReport waterReport=new WaterReport();
            //是否存在，存在则修改，不存在则新增
            WaterReport waterReport_Old=waterReportList.stream().filter(item->item.getWaterId().equals(waterId)).findFirst().orElse(null);
            if(waterReport_Old!=null){
                waterReport.setId(waterReport_Old.getId());
            }else{
                waterReport.setCreateTime(nowDate);
            }

            waterReport.setRecordDate(recordDay);
            waterReport.setWaterId(waterId);
            waterReport.setTotalFlowNow(totalFlowNow);
            waterReport.setDealWaterVolume(dealWaterVolume);
            waterReport.setCodWaterIn(codWaterIn);
            waterReport.setCodWaterOut(codWaterOut);
            waterReport.setAdWaterIn(adWaterIn);
            waterReport.setAdWaterOut(adWaterOut);
            waterReport.setSsWaterIn(ssWaterIn);
            waterReport.setSsWaterOut(ssWaterOut);
            waterReport.setZdWaterIn(zdWaterIn);
            waterReport.setZdWaterOut(zdWaterOut);
            waterReport.setZlWaterIn(zlWaterIn);
            waterReport.setZlWaterOut(zlWaterOut);
            waterReport.setUpdateTime(nowDate);
            waterReports.add(waterReport);
          //  waterReportService.saveOrUpdate(waterReport);
        });
        waterReportService.saveOrUpdateBatch(waterReports);
    }

    private   String getCodeValue(List<ScadaReportRecordVo> recordVoList,String codes,boolean isAvg){
        String result=null;
        List<ScadaReportRecordVo> voList=recordVoList.stream().filter(item->codes.contains(item.getCode())
        ).collect(Collectors.toList());
        if(voList!=null && voList.size()>0){
            if(isAvg){
            result=String.valueOf(voList.stream().mapToDouble(item->Double.parseDouble(item.getAvgValue())).sum());
            }else{
                Double doubleValue=voList.stream().mapToDouble(item->Double.parseDouble(item.getFold())).sum();
                result=String.format("%.2f",doubleValue/10000.0000) ;
            }
        }
        if(result!=null && result.startsWith(".")){
            result="0"+result;
        }
    return result;
}


    /**
     * 保留小数位数
     * @param value
     * @param num
     * @return
     */
    public String getRoundStr(String value,long num){

        if(StringUtils.isEmpty(value)){
            return "";
        }
        int scale=Integer.valueOf(String.valueOf(num));
        if(value.startsWith(".")){
            value="0."+value;
        }
        BigDecimal bd = new BigDecimal(value);
        bd = bd.setScale(scale, RoundingMode.HALF_UP); // 四舍五入到指定的小数位数
        String formattedString = String.format("%." + scale + "f", bd.doubleValue());
        return formattedString;
    }
}




