package com.tofly.scada.controller;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.haiqiu.tools.common.utils.HttpServletUtils;
import com.haiqiu.tools.excel.annotation.Excel;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.common.ExcelTool.ExcelColumn;
import com.tofly.common.ExcelTool.ExcelTool;
import com.tofly.common.constant.CalendarHelper;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.scada.common.BaseController;
import com.tofly.scada.common.DateTimeUtil;
import com.tofly.scada.entity.Scada;
import com.tofly.scada.entity.dto.CustomDisplayDto;
import com.tofly.scada.entity.dto.DeviceQuery;
import com.tofly.scada.entity.dto.ScadaMoreQuery;
import com.tofly.scada.entity.dto.ScadaQuery;
import com.tofly.scada.entity.vo.ScadaVo;
import com.tofly.scada.entity.vo.StatisticsScadaVo;
import com.tofly.scada.service.ScadaService;
import com.tofly.scada.service.StatisticsScadaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.scheduling.annotation.Async;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.*;

/**
 * @author HaiQiu
 * @date 2022/4/25
 */
@Api(tags = "SCADA数据操作")
@RequestMapping("/scada")
@RestController
public class ScadaController extends BaseController {

    @Autowired
    private ScadaService scadaService;

    @Autowired
    private StatisticsScadaService statisticsScadaService;

    @ToFlyAppLog(title = "查询最新SCADA所有数据")
    @ApiOperation(value = "查询最新SCADA所有数据")
    @PostMapping("/all")
    public ResultRespone getData() {
        return decide(scadaService.getListData());
    }

    @ToFlyAppLog(title = "根据设备指标查询最新SCADA所有数据")
    @ApiOperation(value = "根据设备指标查询最新SCADA所有数据")
    @PostMapping("/app/data")
    public ResultRespone getAppData(@RequestBody(required = false) DeviceQuery deviceQuery) {
        return decide(scadaService.getAppData(deviceQuery));
    }


    @ToFlyAppLog(title = "分页查询统计SCADA数据")
    @ApiOperation(value = "分页查询统计SCADA数据")
    @GetMapping("/page")
    public ResultRespone getMonthsData(Page page, ScadaQuery scadaQuery,
                                       @ApiParam("true 分页，false不分页（默认分页）")
                                       @RequestParam(required = false) Boolean isPage
                                     ) {

        isPage=isPage==null?true:isPage;

        Page<StatisticsScadaVo> pageList=statisticsScadaService.getPageData(page, scadaQuery,isPage);
       return decide(pageList);

    }

    @ToFlyAppLog(title = "SCADA数据导出")
    @ApiOperation(value = "分页查询统计SCADA数据")
    @GetMapping("/export")
    public void export(Page page, ScadaQuery scadaQuery,
         @RequestParam(required = false) Boolean isHistory) {
       if(isHistory){
           Page<StatisticsScadaVo> pageList=statisticsScadaService.getPageData(page, scadaQuery,false);
           exportHistory(pageList.getRecords());
        // ExcelUtils.getListToExcel(StatisticsScadaVo.class, pageList.getRecords(), "监控历史导出");

       }else{
           Page<ScadaVo> pageList=statisticsScadaService.geTrendData(page, scadaQuery,false,true);
           exportTrendData(pageList.getRecords());
        // ExcelUtils.getListToExcel(ScadaVo.class, pageList.getRecords(), "趋势分析导出");

       }

    }

    public  void  exportHistory(List<StatisticsScadaVo> list){
        //构建导出的表头，树形结构，包含标题

        List<Map<String,String>> titleList=new ArrayList<>();
        Map<String,String> titleMap= new HashMap<>();
        //表头标题
        String exportTitle="监控历史导出";
        titleMap.put("id","-1");titleMap.put("pid","0");titleMap.put("content",exportTitle);titleMap.put("fielName",null);
        titleList.add(titleMap);

       List<String> listTitle=new ArrayList<>();
        listTitle.add("variableName@指标名称");
        listTitle.add("value@监测值");
        listTitle.add("unit@单位");
        listTitle.add("scadaTime@监测时间");

        listTitle.forEach(item->{
             String id=item.split("@")[0];
            String content=item.split("@")[1];
            String fielName=item.split("@")[0];
            Map<String,String> titleMap1= new HashMap<>();
            titleMap1.put("id",id);
            titleMap1.put("pid","-1");
            titleMap1.put("content",content);
            titleMap1.put("fielName",fielName);
            titleList.add(titleMap1);

        });

        List<Map<String,String>> rowList=new ArrayList<>();
        //循环填充数据
        list.forEach(item->{
            Map<String,String> rowMap= new HashMap<>();
            rowMap.put("variableName",item.getVariableName());
            rowMap.put("value",item.getValue());
            rowMap.put("unit",item.getUnit());
            rowMap.put("scadaTime",item.getScadaTime());
            rowList.add(rowMap);

        });

        ExcelTool excelTool = new ExcelTool(exportTitle);
        List<ExcelColumn>  titleData= null;
        try {
            titleData = excelTool.columnTransformer(titleList,"id","pid","content","fielName","0");
            excelTool.export(exportTitle, excelTool.exportWorkbook(titleData,rowList,true,true), HttpServletUtils.getServletResponse());

        } catch (Exception e) {
            throw new RuntimeException(e);
        }



    }

    public  void  exportTrendData(List<ScadaVo> list){
        //构建导出的表头，树形结构，包含标题

        List<Map<String,String>> titleList=new ArrayList<>();
        Map<String,String> titleMap= new HashMap<>();
        //表头标题
        String exportTitle="趋势分析导出";
        titleMap.put("id","-1");titleMap.put("pid","0");titleMap.put("content",exportTitle);titleMap.put("fielName",null);
        titleList.add(titleMap);

        List<String> listTitle=new ArrayList<>();
        listTitle.add("displayName@指标名称");
        listTitle.add("value@指标值");
        listTitle.add("unit@单位");
        listTitle.add("scadaTime@指标采集时间");

        listTitle.forEach(item->{
            String id=item.split("@")[0];
            String content=item.split("@")[1];
            String fielName=item.split("@")[0];
            Map<String,String> titleMap1= new HashMap<>();
            titleMap1.put("id",id);
            titleMap1.put("pid","-1");
            titleMap1.put("content",content);
            titleMap1.put("fielName",fielName);
            titleList.add(titleMap1);

        });

        List<Map<String,String>> rowList=new ArrayList<>();
        //循环填充数据
        list.forEach(item->{
            Map<String,String> rowMap= new HashMap<>();
            rowMap.put("displayName",item.getDisplayName());
            rowMap.put("value",item.getValue());
            rowMap.put("unit",item.getUnit());
            rowMap.put("scadaTime",item.getScadaTime());
            rowList.add(rowMap);

        });

        ExcelTool excelTool = new ExcelTool(exportTitle);
        List<ExcelColumn>  titleData= null;
        try {
            titleData = excelTool.columnTransformer(titleList,"id","pid","content","fielName","0");
            excelTool.export(exportTitle, excelTool.exportWorkbook(titleData,rowList,true,true), HttpServletUtils.getServletResponse());

        } catch (Exception e) {
            throw new RuntimeException(e);
        }



    }



//    @ToFlyAppLog(title = "查询统计SCADA数据列表")
//    @ApiOperation(value = "查询统计SCADA数据列表")
//    @GetMapping("/list")
//    public ResultRespone getDataList(ScadaQuery scadaQuery){
//        return decide(statisticsScadaService.getDataList(scadaQuery));
//    }

    @ToFlyAppLog(title = "趋势分析统计SCADA数据")
    @ApiOperation(value = "趋势分析统计SCADA数据")
    @GetMapping("/trend/page")
    public ResultRespone geTrendData(Page page, ScadaQuery scadaQuery,
                                     @ApiParam("true 分页，false不分页（默认分页）")
                                     @RequestParam(required = false) Boolean isPage) {

        isPage=isPage==null?true:isPage;

        Page<StatisticsScadaVo> pageList=statisticsScadaService.geTrendData(page, scadaQuery,isPage,false);
       return decide(pageList);

    }


    @ToFlyAppLog(title = "手动更新范围数据，小时，天，月，年，注意时间格式")
    @ApiOperation(value = "手动更新范围数据，小时，天，月，年，注意时间格式")
    @PostMapping("/data/fetch")
    public ResultRespone fetchData(@ApiParam("code编码，非必传,多个逗号隔开") @RequestParam(required = false) String code,
                                   @ApiParam("开始时间:2023-01-01 00:00:00") @RequestParam String start,
                                   @ApiParam("结束时间:2023-10-01 23:59:59") @RequestParam String end,
                                   @ApiParam("统计类型0小时，1按天，2月份，3年份") @RequestParam Integer type) {
        try {


            scadaService.syncDataRangeComm(code,start,end,type);
        }catch (Exception e){
            e.printStackTrace();
            return decide(e.getMessage());

        }

        return decide("更新正在进行中...请稍后再查看结果");
    }

    /*@ToFlyAppLog(title="手动同步指定日期范围内每天/每月/每年的数据，注意时间格式，只需要年月日" )
    @ApiOperation(value = "手动同步指定日期范围内每天的数据,注意时间格式，只需要年月日")
    @PostMapping("/syncDataRangeComm")
    public ResultRespone syncDataComm(
            @ApiParam("code编码，非必传,多个逗号隔开") @RequestParam(required = false) String codes,
            @ApiParam("开始时间:2023-01-01") @RequestParam String startDate,
            @ApiParam("结束时间:2023-10-01") @RequestParam String endDate,
            @ApiParam("统计类型0,小时，1按天，2月份，3年份") @RequestParam Integer dateType
    ) {
        try {

            scadaService.syncDataRangeComm(codes,startDate,endDate,dateType);

        } catch (Exception e) {
            e.printStackTrace();
            return decide(e.getMessage());
        }
        return decide("批量同步成功");
    }*/


   /* @ToFlyAppLog(title="test" )
    @ApiOperation(value = "test")
    @PostMapping("/test")
    public ResultRespone test(
            @ApiParam("code编码，非必传,多个逗号隔开") @RequestParam(required = false) String codes,
            @ApiParam("开始时间:2023-01-01") @RequestParam String startDate,
            @ApiParam("结束时间:2023-10-01") @RequestParam String endDate,
            @ApiParam("统计类型1按天，2月份，3年份") @RequestParam Integer dateType
    ) {
        try {

            scadaService.syncDaysData(null,codes); //同步当天的数据

        } catch (Exception e) {
            e.printStackTrace();
            return decide(e.getMessage());
        }
        return decide("批量同步成功");
    }*/


    @ApiOperation(value = "手动更新水压线数据")
    @PostMapping("/synSyxIndex")
    public ResultRespone synSyxIndex() {
        scadaService.synSyxIndex();

        return decide("更新成功");
    }

    @ApiOperation(value = "手动更新报警数据")
    @PostMapping("/synReport")
    public ResultRespone timingInsertDataNew() {
        scadaService.timingInsertDataNew();
        return decide("更新成功");
    }



    @ToFlyAppLog(title = "手动更新虚拟表数据")
    @ApiOperation(value = "手动更新虚拟表数据")
    @PutMapping("/xn/fetch")
    @Async
    public ResultRespone xnFetchData(String sTime,String eTime) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        try {
            Date startTime = format.parse(sTime);
            Date endTime = format.parse(eTime);
            scadaService.manualSendNetCumulativeFlow(startTime,endTime);
        } catch (Exception e) {
            e.printStackTrace();
            return decide(e.getMessage());
        }


        return decide("更新成功");
    }

    /**
     * 批量新增scada历史数据
     * @param scadaMoreQuery scada历史表
     * @return ResultData
     */
    @ToFlyAppLog(title="批量新增scada历史数据" )
    @ApiOperation(value = "批量新增scada历史数据")
    @PostMapping("/insertBacthHistoryTemp")
    public ResultRespone insertBacthHistoryTemp(@RequestBody ScadaMoreQuery scadaMoreQuery) {
        try {
            statisticsScadaService.insertBacthHistoryTemp(scadaMoreQuery);
        } catch (Exception e) {
            e.printStackTrace();
            return decide(e.getMessage());
        }
        return decide("批量新增成功");
    }





    @GetMapping("/viewScadaData")
    @ApiOperation(value = "查询SCADA指标数据")
    @ToFlyAppLog(title = "查询SCADA指标数据")
    public ResultRespone viewScadaData(CustomDisplayDto dto){
        try {
            return ResultRespone.success(scadaService.viewScadaData(dto));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }

    @GetMapping("/viewScadaDataNew")
    @ApiOperation(value = "查询SCADA指标数据New")
    @ToFlyAppLog(title = "查询SCADA指标数据New")
    public ResultRespone viewScadaDataNew(Page page,CustomDisplayDto dto){
        try {
            return ResultRespone.success(scadaService.viewScadaDataNew(page,dto));
        }catch (Exception e){
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
    }
}
