package com.tofly.scada.handle;

import cn.hutool.core.date.DateUtil;
import com.tofly.common.constant.CalendarHelper;
import com.tofly.custom.service.ScadaReportRecordService;
import com.tofly.scada.service.ScadaHistorySynService;
import com.tofly.scada.service.impl.ScadaServiceImpl;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @author HaiQiu
 * @since 2023/3/24
 */
@Component
public class ScadaReportHandle {
    @Autowired
    private ScadaReportRecordService scadaReportRecordService;


    /**
     * 每10分钟执行一次
     * @return
     */
    @XxlJob("synWaterFlowMin10")
    @Async
    public ReturnT<String> synWaterFlowMin10(){
        //整个日历调整时间
        Integer minutes=10;
        List<String> dates=getListDate(minutes);
        scadaReportRecordService.getWaterFlowCommXXLJobCode("","",minutes,dates.get(0),dates.get(1));
        scadaReportRecordService.getPipeFlowCommXXLJobCode(null,null,minutes,dates.get(0),dates.get(1));
        return ReturnT.SUCCESS;
    }
    /**
     * 每30分钟执行一次
     * @return
     */
    @XxlJob("synWaterFlowMin30")
    @Async
    public ReturnT<String> synWaterFlowMin30(){
        //整个日历调整时间
        Integer minutes=30;
        List<String> dates=getListDate(minutes);
        scadaReportRecordService.getWaterFlowCommXXLJobCode("","",minutes,dates.get(0),dates.get(1));
        scadaReportRecordService.getPipeFlowCommXXLJobCode("","",minutes,dates.get(0),dates.get(1));
        return ReturnT.SUCCESS;
    }
    /**
     * 每60分钟执行一次
     * @return
     */
    @XxlJob("synWaterFlowMin60")
    @Async
    public ReturnT<String> synWaterFlowMin60(){
        //整个日历调整时间
        Integer minutes=60;
        List<String> dates=getListDate(minutes);
        scadaReportRecordService.getWaterFlowCommXXLJobCode("","",minutes,dates.get(0),dates.get(1));
        scadaReportRecordService.getPipeFlowCommXXLJobCode("","",minutes,dates.get(0),dates.get(1));
        Calendar calendar = Calendar.getInstance();
        Date date = new Date();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        String startTime = CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM-dd HH:mm:ss");
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        String startDay= CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM-dd");
        String endTime = CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM-dd HH:mm:ss");
        scadaReportRecordService.getWaterFlowCommXXLJobCode("","",1440,startTime,endTime);
        scadaReportRecordService.getPipeFlowCommXXLJobCode("","",1440,startTime,endTime);

        return ReturnT.SUCCESS;
    }

     @XxlJob("synWaterReport")
     @Async
     public ReturnT<String> synWaterReport(){
         //整个日历调整时间
         Calendar calendar = Calendar.getInstance();
         Date date = new Date();
         calendar.setTime(date);
         String startDay= CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM-dd");
         scadaReportRecordService.getScadaWaterReportXXlJobCode(startDay,startDay);
         return ReturnT.SUCCESS;
     }


    /**
     * 前一天的数据，每天执行一次
     * @return
     */
    @XxlJob("synWaterFlowLastDay")
    @Async
    public ReturnT<String> synWaterFlowLastDay(){
        Calendar calendar = Calendar.getInstance();
        Date date = new Date();
        calendar.setTime(date);
        //延迟一天
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        String startTime = CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM-dd HH:mm:ss");
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        String endTime = CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM-dd HH:mm:ss");
        scadaReportRecordService.getWaterFlowCommXXLJobCode("","",1440,startTime,endTime);
        scadaReportRecordService.getPipeFlowCommXXLJobCode("","",1440,startTime,endTime);
        return ReturnT.SUCCESS;
    }


    /**
     * 前一天的数据，每天执行一次
     * @return
     */
    @XxlJob("synScadaFlowLastDay")
    @Async
    public ReturnT<String> synScadaFlowLastDay(){
        Calendar calendar = Calendar.getInstance();
        Date date = new Date();
        calendar.setTime(date);
        //延迟一天
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        String startTime = CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM-dd");
        //同步昨天的减排报表
        scadaReportRecordService.getScadaWaterReportXXlJobCode(startTime,startTime);

        scadaReportRecordService.getScadaFlowCommXXLJobCode(startTime,startTime);
        return ReturnT.SUCCESS;
    }


   /* *//**
     * 当天的数据，每1小时执行1次
     * @return
     *//*
    @XxlJob("synWaterFlowDay")
    @Async
    public ReturnT<String> synWaterFlowDay(){
        Calendar calendar = Calendar.getInstance();
        Date date = new Date();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        String startTime = CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM-dd HH:mm:ss");
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        String endTime = CalendarHelper.getFormatDate(calendar.getTime(),"yyyy-MM-dd HH:mm:ss");
        scadaReportRecordService.getWaterFlowCommXXLJobCode("","",1440,startTime,endTime);
        return ReturnT.SUCCESS;
    }*/

    public List<String> getListDate(Integer minutes) {
        List<String> list=new ArrayList<>();
        Calendar calendar = Calendar.getInstance();
        Date date = new Date();
        calendar.setTime(date);
        String endTime = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        calendar.add(Calendar.MINUTE, -minutes*2);
        String startTime = DateUtil.format(calendar.getTime(), "yyyy-MM-dd HH:mm:ss");
        list.add(startTime);
        list.add(endTime);
        return  list;
    }


}
