package com.tofly.scada.service.impl;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.constant.CalendarHelper;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.scada.service.WaterReportService;
import com.tofly.water.entity.WaterReport;
import com.tofly.water.entity.vo.WaterReportDayVo;
import com.tofly.water.mapper.WaterReportMapper;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
*/

@Service("waterReportService")
public class WaterReportServiceImpl extends ServiceImpl<WaterReportMapper, WaterReport> implements WaterReportService
{

    @Override
    public WaterReport getByDate(String recordDate) {
       Date date= CalendarHelper.ConvertStrToDate(recordDate,"yyyy-MM-dd");
      return  super.getOne(Wrappers.<WaterReport>lambdaQuery()
                .eq(WaterReport::getRecordDate,date));
    }

    @Override
    public Boolean saveData(WaterReport waterReport) {
        String recordDate=CalendarHelper.getFormatDate(waterReport.getRecordDate(),"yyyy-MM-dd");
        WaterReport vo=getByDate(recordDate);
        if(vo!=null && vo.getId()>0){
            throw new RuntimeException("记录日期："+recordDate+"数据已经存在，如需操作，请修改");
        }
       return super.save(waterReport);
    }


    @Override
    public WaterReportDayVo reportMonth(String yearAndMonth, Long waterId) {
        String beginTime=yearAndMonth+"-01";
        String simpleDateFormat="yyyy-MM-dd";
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(CalendarHelper.ConvertStrToDate(beginTime,simpleDateFormat));

        //倒退一个月最后一天时间
        calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
         String endTime=CalendarHelper.getFormatDate(calendar.getTime(),simpleDateFormat);

        WaterReportDayVo vo=new WaterReportDayVo();
        List<WaterReport> list=this.baseMapper.getReportMonth(beginTime,endTime,waterId);
        vo.setList(list);

        List<WaterReport> listFooter=new ArrayList<>();
        if(list!=null && list.size()>0){
            //日处理水量
             String dealWaterVolume_Double_min="";
            String dealWaterVolume_Double_max="";
            String dealWaterVolume_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getDealWaterVolume())).count()>0){
                dealWaterVolume_Double_min=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getDealWaterVolume())).mapToDouble(item->Double.parseDouble(item.getDealWaterVolume())).min().getAsDouble());
                dealWaterVolume_Double_max=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getDealWaterVolume())).mapToDouble(item->Double.parseDouble(item.getDealWaterVolume())).max().getAsDouble());
                dealWaterVolume_Double_avg=getAvgStr(list.stream().filter(item-> StringUtil.isNotEmpty(item.getDealWaterVolume())).mapToDouble(item->Double.parseDouble(item.getDealWaterVolume())).average().getAsDouble());
            }
            String totalFlowNow_Double_min="";
            String totalFlowNow_Double_max="";
            String totalFlowNow_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getTotalFlowNow())).count()>0){
                totalFlowNow_Double_min=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getTotalFlowNow())).mapToDouble(item->Double.parseDouble(item.getTotalFlowNow())).min().getAsDouble());
                totalFlowNow_Double_max=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getTotalFlowNow())).mapToDouble(item->Double.parseDouble(item.getTotalFlowNow())).max().getAsDouble());
                totalFlowNow_Double_avg=getAvgStr(list.stream().filter(item-> StringUtil.isNotEmpty(item.getTotalFlowNow())).mapToDouble(item->Double.parseDouble(item.getTotalFlowNow())).average().getAsDouble());
            }


            String codIn_Double_min="";
            String codIn_Double_max="";
            String codIn_Double_avg="";
            //cod
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getCodWaterIn())).count()>0){

             codIn_Double_min=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getCodWaterIn())).mapToDouble(item->Double.parseDouble(item.getCodWaterIn())).min().getAsDouble());
             codIn_Double_max=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getCodWaterIn())).mapToDouble(item->Double.parseDouble(item.getCodWaterIn())).max().getAsDouble());
             codIn_Double_avg=getAvgStr(list.stream().filter(item-> StringUtil.isNotEmpty(item.getCodWaterIn())).mapToDouble(item->Double.parseDouble(item.getCodWaterIn())).average().getAsDouble());
            }

             String codOut_Double_min="";
             String codOut_Double_max="";
             String codOut_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getCodWaterOut())).count()>0){
                 codOut_Double_min=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getCodWaterOut())).mapToDouble(item->Double.parseDouble(item.getCodWaterOut())).min().getAsDouble());
                 codOut_Double_max=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getCodWaterOut())).mapToDouble(item->Double.parseDouble(item.getCodWaterOut())).max().getAsDouble());
                 codOut_Double_avg=getAvgStr(list.stream().filter(item-> StringUtil.isNotEmpty(item.getCodWaterOut())).mapToDouble(item->Double.parseDouble(item.getCodWaterOut())).average().getAsDouble());

            }

            //bod
            String bodIn_Double_min="";
            String bodIn_Double_max="";
            String bodIn_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getBodWaterIn())).count()>0){
                 bodIn_Double_min=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getBodWaterIn())).mapToDouble(item->Double.parseDouble(item.getBodWaterIn())).min().getAsDouble());
                 bodIn_Double_max=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getBodWaterIn())).mapToDouble(item->Double.parseDouble(item.getBodWaterIn())).max().getAsDouble());
                 bodIn_Double_avg=getAvgStr(list.stream().filter(item-> StringUtil.isNotEmpty(item.getBodWaterIn())).mapToDouble(item->Double.parseDouble(item.getBodWaterIn())).average().getAsDouble());

            }
            String bodOut_Double_min="";
            String bodOut_Double_max="";
            String bodOut_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getBodWaterOut())).count()>0){
             bodOut_Double_min=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getBodWaterOut())).mapToDouble(item->Double.parseDouble(item.getBodWaterOut())).min().getAsDouble());
             bodOut_Double_max=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getBodWaterOut())).mapToDouble(item->Double.parseDouble(item.getBodWaterOut())).max().getAsDouble());
             bodOut_Double_avg=getAvgStr(list.stream().filter(item-> StringUtil.isNotEmpty(item.getBodWaterOut())).mapToDouble(item->Double.parseDouble(item.getBodWaterOut())).average().getAsDouble());

            }

             //ad 氨氮
            String adIn_Double_min="";
            String adIn_Double_max="";
            String adIn_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getAdWaterIn())).count()>0) {
                 adIn_Double_min = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getAdWaterIn())).mapToDouble(item -> Double.parseDouble(item.getAdWaterIn())).min().getAsDouble());
                 adIn_Double_max = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getAdWaterIn())).mapToDouble(item -> Double.parseDouble(item.getAdWaterIn())).max().getAsDouble());
                 adIn_Double_avg = getAvgStr(list.stream().filter(item -> StringUtil.isNotEmpty(item.getAdWaterIn())).mapToDouble(item -> Double.parseDouble(item.getAdWaterIn())).average().getAsDouble());
            }
            String adOut_Double_min="";
            String adOut_Double_max="";
            String adOut_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getAdWaterOut())).count()>0) {
                 adOut_Double_min = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getAdWaterOut())).mapToDouble(item -> Double.parseDouble(item.getAdWaterOut())).min().getAsDouble());
                 adOut_Double_max = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getAdWaterOut())).mapToDouble(item -> Double.parseDouble(item.getAdWaterOut())).max().getAsDouble());
                 adOut_Double_avg = getAvgStr(list.stream().filter(item -> StringUtil.isNotEmpty(item.getAdWaterOut())).mapToDouble(item -> Double.parseDouble(item.getAdWaterOut())).average().getAsDouble());
            }
           //ss
            String ssIn_Double_min="";
            String ssIn_Double_max="";
            String ssIn_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getSsWaterIn())).count()>0) {
             ssIn_Double_min=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getSsWaterIn())).mapToDouble(item->Double.parseDouble(item.getSsWaterIn())).min().getAsDouble());
             ssIn_Double_max=String.valueOf(list.stream().filter(item-> StringUtil.isNotEmpty(item.getSsWaterIn())).mapToDouble(item->Double.parseDouble(item.getSsWaterIn())).max().getAsDouble());
             ssIn_Double_avg=getAvgStr(list.stream().filter(item-> StringUtil.isNotEmpty(item.getSsWaterIn())).mapToDouble(item->Double.parseDouble(item.getSsWaterIn())).average().getAsDouble());
            }
            //ss
            String ssOut_Double_min="";
            String ssOut_Double_max="";
            String ssOut_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getSsWaterOut())).count()>0) {
                 ssOut_Double_min = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getSsWaterOut())).mapToDouble(item -> Double.parseDouble(item.getSsWaterOut())).min().getAsDouble());
                 ssOut_Double_max = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getSsWaterOut())).mapToDouble(item -> Double.parseDouble(item.getSsWaterOut())).max().getAsDouble());
                 ssOut_Double_avg = getAvgStr(list.stream().filter(item -> StringUtil.isNotEmpty(item.getSsWaterOut())).mapToDouble(item -> Double.parseDouble(item.getSsWaterOut())).average().getAsDouble());
            }
             //总磷
            String zlIn_Double_min="";
            String zlIn_Double_max="";
            String zlIn_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getZlWaterIn())).count()>0) {
                 zlIn_Double_min = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZlWaterIn())).mapToDouble(item -> Double.parseDouble(item.getZlWaterIn())).min().getAsDouble());
                 zlIn_Double_max = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZlWaterIn())).mapToDouble(item -> Double.parseDouble(item.getZlWaterIn())).max().getAsDouble());
                 zlIn_Double_avg = getAvgStr(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZlWaterIn())).mapToDouble(item -> Double.parseDouble(item.getZlWaterIn())).average().getAsDouble());
            }
            String zlOut_Double_min="";
            String zlOut_Double_max="";
            String zlOut_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getZlWaterOut())).count()>0) {
                 zlOut_Double_min = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZlWaterOut())).mapToDouble(item -> Double.parseDouble(item.getZlWaterOut())).min().getAsDouble());
                 zlOut_Double_max = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZlWaterOut())).mapToDouble(item -> Double.parseDouble(item.getZlWaterOut())).max().getAsDouble());
                 zlOut_Double_avg =getAvgStr(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZlWaterOut())).mapToDouble(item -> Double.parseDouble(item.getZlWaterOut())).average().getAsDouble());
            }
             //总氮
            String zdIn_Double_min="";
            String zdIn_Double_max="";
            String zdIn_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getZdWaterIn())).count()>0) {
                 zdIn_Double_min = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZdWaterIn())).mapToDouble(item -> Double.parseDouble(item.getZdWaterIn())).min().getAsDouble());
                 zdIn_Double_max = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZdWaterIn())).mapToDouble(item -> Double.parseDouble(item.getZdWaterIn())).max().getAsDouble());
                 zdIn_Double_avg = getAvgStr(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZdWaterIn())).mapToDouble(item -> Double.parseDouble(item.getZdWaterIn())).average().getAsDouble());
            }
            String zdOut_Double_min="";
            String zdOut_Double_max="";
            String zdOut_Double_avg="";
            if(list.stream().filter(item-> StringUtil.isNotEmpty(item.getZdWaterOut())).count()>0) {
                 zdOut_Double_min = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZdWaterOut())).mapToDouble(item -> Double.parseDouble(item.getZdWaterOut())).min().getAsDouble());
                 zdOut_Double_max = String.valueOf(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZdWaterOut())).mapToDouble(item -> Double.parseDouble(item.getZdWaterOut())).max().getAsDouble());
                 zdOut_Double_avg = getAvgStr(list.stream().filter(item -> StringUtil.isNotEmpty(item.getZdWaterOut())).mapToDouble(item -> Double.parseDouble(item.getZdWaterOut())).average().getAsDouble());
            }
           //数据处理
            WaterReport waterReportMin=new WaterReport();
            WaterReport waterReportMax=new WaterReport();
            WaterReport waterReportAvg=new WaterReport();
            waterReportMin.setRecordDay("最小值");
            waterReportMax.setRecordDay("最大值");
            waterReportAvg.setRecordDay("平均值");

            waterReportMin.setDealWaterVolume(dealWaterVolume_Double_min);
            waterReportMax.setDealWaterVolume(dealWaterVolume_Double_max);
            waterReportAvg.setDealWaterVolume(dealWaterVolume_Double_avg);

            waterReportMin.setTotalFlowNow(totalFlowNow_Double_min);
            waterReportMax.setTotalFlowNow(totalFlowNow_Double_max);
            waterReportAvg.setTotalFlowNow(totalFlowNow_Double_avg);

            waterReportMin.setCodWaterIn(codIn_Double_min);
            waterReportMax.setCodWaterIn(codIn_Double_max);
            waterReportAvg.setCodWaterIn(codIn_Double_avg);

            waterReportMin.setCodWaterOut(codOut_Double_min);
            waterReportMax.setCodWaterOut(codOut_Double_max);
            waterReportAvg.setCodWaterOut(codOut_Double_avg);

            waterReportMin.setBodWaterIn(bodIn_Double_min);
            waterReportMax.setBodWaterIn(bodIn_Double_max);
            waterReportAvg.setBodWaterIn(bodIn_Double_avg);

            waterReportMin.setBodWaterOut(bodOut_Double_min);
            waterReportMax.setBodWaterOut(bodOut_Double_max);
            waterReportAvg.setBodWaterOut(bodOut_Double_avg);

            waterReportMin.setAdWaterIn(adIn_Double_min);
            waterReportMax.setAdWaterIn(adIn_Double_max);
            waterReportAvg.setAdWaterIn(adIn_Double_avg);

            waterReportMin.setAdWaterOut(adOut_Double_min);
            waterReportMax.setAdWaterOut(adOut_Double_max);
            waterReportAvg.setAdWaterOut(adOut_Double_avg);

            waterReportMin.setSsWaterIn(ssIn_Double_min);
            waterReportMax.setSsWaterIn(ssIn_Double_max);
            waterReportAvg.setSsWaterIn(ssIn_Double_avg);

            waterReportMin.setSsWaterOut(ssOut_Double_min);
            waterReportMax.setSsWaterOut(ssOut_Double_max);
            waterReportAvg.setSsWaterOut(ssOut_Double_avg);

            waterReportMin.setZlWaterIn(zlIn_Double_min);
            waterReportMax.setZlWaterIn(zlIn_Double_max);
            waterReportAvg.setZlWaterIn(zlIn_Double_avg);

            waterReportMin.setZlWaterOut(zlOut_Double_min);
            waterReportMax.setZlWaterOut(zlOut_Double_max);
            waterReportAvg.setZlWaterOut(zlOut_Double_avg);

            waterReportMin.setZdWaterIn(zdIn_Double_min);
            waterReportMax.setZdWaterIn(zdIn_Double_max);
            waterReportAvg.setZdWaterIn(zdIn_Double_avg);

            waterReportMin.setZdWaterOut(zdOut_Double_min);
            waterReportMax.setZdWaterOut(zdOut_Double_max);
            waterReportAvg.setZdWaterOut(zdOut_Double_avg);

            listFooter.add(waterReportMin);
            listFooter.add(waterReportMax);
            listFooter.add(waterReportAvg);

        }

        vo.setListFooter(listFooter);
        return vo;
    }

    @Override
    public boolean saveDataBatch(List<WaterReport> waterReportList) {
        waterReportList.forEach(item->{
            item.setUpdateUser(SecurityUtils.getUserId());
            item.setUpdateTime(new Date());
            if(item.getId()==null){
                item.setCreateTime(new Date());
                item.setCreateUser(SecurityUtils.getUserId());
            }
        });
        return super.saveOrUpdateBatch(waterReportList);
    }


    public String getAvgStr(double num){
        DecimalFormat df = new DecimalFormat("#.00");
        return  df.format(num);

   }
}




