/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.water.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.scada.common.BaseController;
import com.tofly.scada.service.WaterReportService;
import com.tofly.water.entity.WaterReport;
import com.tofly.water.entity.vo.WaterReportDayVo;
import com.tofly.water.entity.vo.WaterReportDto;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;

/**
 * 污水厂排污报表
 *
 * @author JF
 * @date Wed Aug 03 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/waterReport")
@Api(tags="污水厂减排报表")
public class WaterReportController extends BaseController {

    private final WaterReportService waterReportService;

    /**
    * 污水厂减排报表_日报
    * @param yearAndMonth 年月
    * @return
    */
    @GetMapping("/reportMonth" )
    @ApiOperation(value = "污水厂减排报表_日报")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "yearAndMonth", value = "年月，yyyy-MM", required = true, dataType = "String"),
            @ApiImplicitParam(name = "waterId", value = "水厂Id", required = true, dataType = "Long")
    })

    public ResultRespone<WaterReportDayVo> reportMonth(String yearAndMonth,Long waterId) {
      return ResultRespone.success(waterReportService.reportMonth(yearAndMonth,waterId));
    }


    /**
     * 通过id查询污水厂减排报表
     * @param id id
     * @return ResultData
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过id查询污水厂减排报表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(waterReportService.getById(id));
    }


    /**
     * 通过记录日期查询数据
     * @param recordDate id
     * @return ResultData
     */
    @GetMapping("/getByDate" )
    @ApiOperation(value = "通过日期查询污水厂减排报表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "recordDate", value = "记录日期yyyy-MM-dd", required = true, dataType = "String")
    })
    public ResultRespone getByDate(String recordDate) {
        return ResultRespone.success(waterReportService.getByDate(recordDate));
    }

    /**
     * 污水厂减排报表
     * @param waterReport 行政区划
     * @return ResultData
     */

    @ApiOperation(value = "新增污水厂减排报表")
    @PostMapping
    public ResultRespone saveData(@RequestBody @Valid WaterReport waterReport) {
      return ResultRespone.success(waterReportService.saveData(waterReport));
    }

    /**
     * 批量新增/修改污水厂减排报表
     * @param waterReportDto 行政区划
     * @return ResultData
     */

    @ApiOperation(value = "批量新增/修改污水厂减排报表")
    @PostMapping("batchSave")
    public ResultRespone saveDataBatch(@RequestBody @Valid WaterReportDto waterReportDto) {
        return ResultRespone.success(waterReportService.saveDataBatch(waterReportDto.getWaterReportList()));
    }
    /**
     * 修改污水厂排污报表
     * @param waterReport 污水厂排污报表
     * @return ResultData
     */

    @ApiOperation(value = "修改污水厂减排报表")
    @PutMapping
    public ResultRespone updateById(@RequestBody WaterReport waterReport) {
      return ResultRespone.success(waterReportService.updateById(waterReport));
    }

    /**
     * 通过id删除污水厂减排报表
     * @param id id
     * @return ResultData
     */
    @ApiOperation(value = "通过ID删除污水厂减排报表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(waterReportService.removeById(id));
    }

    /**
     * 通过id批量删除污水厂减排报表
     * @param ids id
     * @return ResultData
     */
    @ToFlyAppLog(title="通过id批量删除污水厂减排报表" )
    @ApiOperation(value = "通过ID批量删除污水厂减排报表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(waterReportService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
